@@ -1,17 +1,13 @@
 //
 // Client processing code for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -112,7 +108,7 @@ _papplClientCreateTempFile(
   }
 
   // Write the data to a temporary file...
-  if ((fd = cupsTempFd(NULL, NULL, tempfile, sizeof(tempfile))) < 0)
+  if ((fd = cupsCreateTempFd(NULL, NULL, tempfile, sizeof(tempfile))) < 0)
   {
     papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to create temporary file: %s", strerror(errno));
     return (NULL);
@@ -217,8 +213,8 @@ _papplClientProcessHTTP(
   // Parse the request line...
   if (http_state == HTTP_STATE_ERROR)
   {
-    if (httpError(client->http) != EPIPE && httpError(client->http))
-      papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "Bad request line (%s).", strerror(httpError(client->http)));
+    if (httpGetError(client->http) != EPIPE && httpGetError(client->http))
+      papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "Bad request line (%s).", strerror(httpGetError(client->http)));
 
     return (false);
   }
@@ -315,7 +311,7 @@ _papplClientProcessHTTP(
 
       if (!httpSetEncryption(client->http, HTTP_ENCRYPTION_REQUIRED))
       {
-	papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsLastErrorString());
+	papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsGetErrorString());
 	return (false);
       }
 
@@ -425,7 +421,7 @@ _papplClientProcessHTTP(
 	  {
 	    if (ipp_state == IPP_STATE_ERROR)
 	    {
-	      papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "IPP read error (%s).", cupsLastErrorString());
+	      papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "IPP read error (%s).", cupsGetErrorString());
 	      papplClientRespond(client, HTTP_STATUS_BAD_REQUEST, NULL, NULL, 0, 0);
 	      return (false);
 	    }
@@ -655,7 +651,7 @@ _papplClientRun(
 
 	if (!httpSetEncryption(client->http, HTTP_ENCRYPTION_ALWAYS))
 	{
-          papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsLastErrorString());
+          papplLogClient(client, PAPPL_LOGLEVEL_ERROR, "Unable to encrypt connection: %s", cupsGetErrorString());
 	  break;
         }
 
@@ -671,7 +667,7 @@ _papplClientRun(
     _papplClientCleanTempFiles(client);
   }
 
-  // Close the conection to the client and return...
+  // Close the connection to the client and return...
   _papplClientDelete(client);
 
   return (NULL);
