@@ -1,17 +1,13 @@
 //
 // Network device support code for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 // Copyright © 2007-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "device-private.h"
 #include "dnssd-private.h"
 #include "snmp-private.h"
@@ -216,17 +212,17 @@ static void		utf16_to_utf8(char *dst, const unsigned char *src, size_t srcsize,
 
 
 //
-// '_papplDeviceAddNetworkSchemes()' - Add all of the supported network schemes.
+// '_papplDeviceAddNetworkSchemesNoLock()' - Add all of the supported network schemes.
 //
 
 void
-_papplDeviceAddNetworkSchemes(void)
+_papplDeviceAddNetworkSchemesNoLock(void)
 {
 #ifdef HAVE_DNSSD
-  papplDeviceAddScheme2("dnssd", PAPPL_DEVTYPE_DNS_SD, pappl_dnssd_list, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
+  _papplDeviceAddSchemeNoLock("dnssd", PAPPL_DEVTYPE_DNS_SD, pappl_dnssd_list, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
 #endif // HAVE_DNSSD
-  papplDeviceAddScheme2("snmp", PAPPL_DEVTYPE_SNMP, pappl_snmp_list, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
-  papplDeviceAddScheme2("socket", PAPPL_DEVTYPE_SOCKET, NULL, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
+  _papplDeviceAddSchemeNoLock("snmp", PAPPL_DEVTYPE_SNMP, pappl_snmp_list, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
+  _papplDeviceAddSchemeNoLock("socket", PAPPL_DEVTYPE_SOCKET, NULL, pappl_socket_open, pappl_socket_close, pappl_socket_read, pappl_socket_write, pappl_socket_status, pappl_socket_supplies, pappl_socket_getid);
 }
 
 
@@ -315,6 +311,20 @@ pappl_dnssd_free(_pappl_dns_sd_dev_t *d)// I - Device
   free(d->device_id);
   free(d->uuid);
   pthread_mutex_destroy(&d->mutex);
+
+#ifdef HAVE_MDNSRESPONDER
+  if (d->ref)
+    DNSServiceRefDeallocate(d->ref);
+
+#elif defined(HAVE_AVAHI)
+  if (d->ref)
+  {
+    _papplDNSSDLock();
+    avahi_record_browser_free(d->ref);
+    _papplDNSSDUnlock();
+  }
+#endif // HAVE_MDNSRESPONDER
+
   free(d);
 }
 
@@ -422,7 +432,8 @@ pappl_dnssd_list(
   bool			ret = false;	// Return value
   cups_array_t		*devices;	// DNS-SD devices
   _pappl_dns_sd_dev_t	*device;	// Current DNS-SD device
-  char			device_name[1024],
+  char			device_id[1024],// IEEE-1284 device ID
+			device_name[1024],
 					// Network device name
 			device_uri[1024];
 					// Network device URI
@@ -433,6 +444,7 @@ pappl_dnssd_list(
   DNSServiceRef		pdl_ref;	// Browse reference for _pdl-datastream._tcp
 #  else
   AvahiServiceBrowser	*pdl_ref;	// Browse reference for _pdl-datastream._tcp
+  _pappl_dns_sd_t	dnssd;		// DNS-SD service
 #  endif // HAVE_MDNSRESPONDER
 
 
@@ -450,16 +462,26 @@ pappl_dnssd_list(
   {
     _papplDeviceError(err_cb, err_data, "Unable to create service browser: %s (%d).", _papplDNSSDStrError(error), error);
     cupsArrayDelete(devices);
+    _papplDNSSDUnlock();
     return (ret);
   }
 
   _PAPPL_DEBUG("pappl_dnssd_find: pdl_ref=%p (after)\n", pdl_ref);
 
 #  else
-  if ((pdl_ref = avahi_service_browser_new(_papplDNSSDInit(NULL), AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, "_pdl-datastream._tcp", NULL, 0, pappl_dnssd_browse_cb, devices)) == NULL)
+  if ((dnssd = _papplDNSSDInit(NULL)) == NULL)
+  {
+    _papplDeviceError(err_cb, err_data, "Unable to create DNSSD service.");
+    cupsArrayDelete(devices);
+    _papplDNSSDUnlock();
+    return (ret);
+  }
+
+  if ((pdl_ref = avahi_service_browser_new(dnssd, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, "_pdl-datastream._tcp", NULL, 0, pappl_dnssd_browse_cb, devices)) == NULL)
   {
     _papplDeviceError(err_cb, err_data, "Unable to create service browser.");
     cupsArrayDelete(devices);
+    _papplDNSSDUnlock();
     return (ret);
   }
 #  endif // HAVE_MDNSRESPONDER
@@ -502,7 +524,14 @@ pappl_dnssd_list(
     else
       httpAssembleURI(HTTP_URI_CODING_ALL, device_uri, sizeof(device_uri), "dnssd", NULL, device->fullName, 0, "/");
 
-    if ((*cb)(device_name, device_uri, device->device_id, data))
+    pthread_mutex_lock(&device->mutex);
+    if (device->device_id)
+      papplCopyString(device_id, device->device_id, sizeof(device_id));
+    else
+      device_id[0] = '\0';
+    pthread_mutex_unlock(&device->mutex);
+
+    if ((*cb)(device_name, device_uri, device_id, data))
     {
       ret = true;
       break;
@@ -543,7 +572,7 @@ pappl_dnssd_query_cb(
     AvahiProtocol          protocol,	// I - Network protocol (unused)
     AvahiBrowserEvent      event,	// I - What happened?
     const char             *fullName,	// I - Service name
-    uint16_t               rrclass,	// I - Record clasa
+    uint16_t               rrclass,	// I - Record class
     uint16_t               rrtype,	// I - Record type
     const void             *rdata,	// I - TXT record
     size_t                 rdlen,	// I - Length of TXT record
@@ -1679,7 +1708,7 @@ pappl_socket_open(
   snprintf(port_str, sizeof(port_str), "%d", sock->port);
   if ((sock->list = httpAddrGetList(sock->host, AF_UNSPEC, port_str)) == NULL)
   {
-    papplDeviceError(device, "Unable to lookup '%s:%d': %s", sock->host, sock->port, cupsLastErrorString());
+    papplDeviceError(device, "Unable to lookup '%s:%d': %s", sock->host, sock->port, cupsGetErrorString());
     goto error;
   }
 
@@ -1688,7 +1717,7 @@ pappl_socket_open(
 
   if (sock->fd < 0)
   {
-    papplDeviceError(device, "Unable to connect to '%s:%d': %s", sock->host, sock->port, cupsLastErrorString());
+    papplDeviceError(device, "Unable to connect to '%s:%d': %s", sock->host, sock->port, cupsGetErrorString());
     goto error;
   }
 
