@@ -1,17 +1,13 @@
 //
 // Job IPP processing for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -49,7 +45,7 @@ _papplJobCopyAttributesNoLock(
     pappl_client_t *client,		// I - Client
     cups_array_t   *ra)			// I - requested-attributes
 {
-  _papplCopyAttributes(client->response, job->attrs, ra, IPP_TAG_JOB, 0);
+  _papplCopyAttributes(client->response, job->attrs, ra, IPP_TAG_JOB, false);
 
   if (!ra || cupsArrayFind(ra, "date-time-at-creation"))
     ippAddDate(client->response, IPP_TAG_JOB, "date-time-at-creation", ippTimeToDate(job->created));
@@ -152,8 +148,6 @@ _papplJobCopyDocumentData(
 
   while ((bytes = httpRead(client->http, buffer, sizeof(buffer))) > 0)
   {
-    papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "Read %d bytes...", (int)bytes);
-
     if (write(job->fd, buffer, (size_t)bytes) < bytes)
     {
       int error = errno;		// Write error
@@ -244,9 +238,6 @@ _papplJobCopyDocumentData(
     client->system->clean_time = time(NULL) + 60;
   _papplRWUnlock(client->system);
 
-  _papplRWUnlock(job);
-  _papplRWUnlock(client->printer);
-
   ra = cupsArrayNew((cups_array_cb_t)strcmp, NULL, NULL, 0, NULL, NULL);
   cupsArrayAdd(ra, "job-id");
   cupsArrayAdd(ra, "job-state");
@@ -255,11 +246,14 @@ _papplJobCopyDocumentData(
 
   _papplJobCopyAttributesNoLock(job, client, ra);
   cupsArrayDelete(ra);
+
+  _papplRWUnlock(job);
+  _papplRWUnlock(client->printer);
 }
 
 
 //
-// '_papplJobCopyStateNoLock()' - Copy the job-state-xxx sttributes.
+// '_papplJobCopyStateNoLock()' - Copy the job-state-xxx attributes.
 //
 
 void
@@ -511,6 +505,7 @@ _papplJobValidateDocumentAttributes(
     memset(header, 0, sizeof(header));
     headersize = httpPeek(client->http, (char *)header, sizeof(header));
 
+    _papplRWLockRead(client->system);
     if (!memcmp(header, "%PDF", 4))
       format = "application/pdf";
     else if (!memcmp(header, "%!", 2))
@@ -528,6 +523,8 @@ _papplJobValidateDocumentAttributes(
     else
       format = NULL;
 
+    _papplRWUnlock(client->system);
+
     papplLogClient(client, PAPPL_LOGLEVEL_DEBUG, "Auto-type header: %02X%02X%02X%02X%02X%02X%02X%02X... format: %s\n", header[0], header[1], header[2], header[3], header[4], header[5], header[6], header[7], format ? format : "unknown");
 
     if (format)
@@ -677,9 +674,13 @@ ipp_get_job_attributes(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
+  _papplRWLockRead(job);
+
   ra = ippCreateRequestedArray(client->request);
   _papplJobCopyAttributesNoLock(job, client, ra);
   cupsArrayDelete(ra);
+
+  _papplRWUnlock(job);
 }
 
 
@@ -843,7 +844,7 @@ ipp_send_document(
   // Then finish getting the document data and process things...
   _papplRWLockWrite(client->printer);
 
-  _papplCopyAttributes(job->attrs, client->request, NULL, IPP_TAG_JOB, 0);
+  _papplCopyAttributes(job->attrs, client->request, NULL, IPP_TAG_JOB, false);
 
   if ((attr = ippFindAttribute(job->attrs, "document-format-detected", IPP_TAG_MIMETYPE)) != NULL)
     job->format = ippGetString(attr, 0, NULL);
