@@ -1,16 +1,12 @@
 //
 // Job processing (printing) functions for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -49,7 +45,7 @@ papplJobCreatePrintOptions(
   pappl_printer_t	*printer = job->printer;
 					// Printer
   const char		*raster_type;	// Raster type for output
-#if CUPS_VERSION_MAJOR < 3
+#if CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
   static const char * const media_positions[] =
   {					// "media-source" to MediaPosition mapping
     "auto",
@@ -111,7 +107,7 @@ papplJobCreatePrintOptions(
     CUPS_ORIENT_180,
     CUPS_ORIENT_0
   };
-#endif // CUPS_VERSION_MAJOR < 3
+#endif // CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
   static const char * const sheet_back[] =
   {					// "pwg-raster-document-sheet-back values
     "normal",
@@ -132,10 +128,7 @@ papplJobCreatePrintOptions(
   _papplRWLockRead(printer);
 
   // copies
-  if ((attr = ippFindAttribute(job->attrs, "copies", IPP_TAG_INTEGER)) != NULL)
-    options->copies = ippGetInteger(attr, 0);
-  else
-    options->copies = 1;
+  options->copies = job->copies;
 
   // finishings
   options->finishings = PAPPL_FINISHINGS_NONE;
@@ -415,7 +408,7 @@ papplJobCreatePrintOptions(
     memcpy(options->dither, printer->driver_data.gdither, sizeof(options->dither));
 
   // Generate the raster header...
-#if CUPS_VERSION_MAJOR < 3
+#if CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
   cupsRasterInitPWGHeader(&options->header, pwgMediaForPWG(options->media.size_name), raster_type, options->printer_resolution[0], options->printer_resolution[1], _papplSidesString(options->sides), sheet_back[printer->driver_data.duplex]);
   for (i = 0; i < (int)(sizeof(media_positions) / sizeof(media_positions[0])); i ++)
   {
@@ -431,14 +424,14 @@ papplJobCreatePrintOptions(
     options->header.CutMedia = CUPS_CUT_PAGE;
   options->header.Orientation = orientations[options->orientation_requested - IPP_ORIENT_PORTRAIT];
   options->header.cupsInteger[CUPS_RASTER_PWG_TotalPageCount] = (unsigned)options->copies * options->num_pages;
+  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxBottom] = options->header.cupsHeight - (unsigned)options->media.bottom_margin * options->header.HWResolution[1] / 2540 - 1;
   options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxLeft]   = (unsigned)options->media.left_margin * options->header.HWResolution[0] / 2540;
-  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxTop]    = options->header.cupsHeight - (unsigned)options->media.top_margin * options->header.HWResolution[1] / 2540;
-  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxRight]  = options->header.cupsWidth - (unsigned)options->media.right_margin * options->header.HWResolution[0] / 2540;
-  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxBottom] = (unsigned)options->media.bottom_margin * options->header.HWResolution[1] / 2540;
+  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxRight]  = options->header.cupsWidth - (unsigned)options->media.right_margin * options->header.HWResolution[0] / 2540 - 1;
+  options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxTop]    = (unsigned)options->media.top_margin * options->header.HWResolution[1] / 2540;
   options->header.cupsInteger[CUPS_RASTER_PWG_PrintQuality]   = options->print_quality;
 
-#else // CUPS 3.x has a new API for this...
-  cups_size_t	media;			// CUPS media value
+#else // CUPS 2.5/3.x have a new API for this...
+  cups_media_t	media;			// CUPS media value
 
   memset(&media, 0, sizeof(media));
 
@@ -454,7 +447,7 @@ papplJobCreatePrintOptions(
   media.top    = options->media.top_margin;
 
   cupsRasterInitHeader(&options->header, &media, _papplContentString(options->print_content_optimize), options->print_quality, /*intent*/NULL, options->orientation_requested, _papplSidesString(options->sides), raster_type, options->printer_resolution[0], options->printer_resolution[1], sheet_back[printer->driver_data.duplex]);
-#endif // CUPS_VERSION_MAJOR < 3
+#endif // CUPS_VERSION_MAJOR < 3 && CUPS_VERSION_MINOR < 5
 
   // Log options...
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.cupsWidth=%u", options->header.cupsWidth);
@@ -466,7 +459,7 @@ papplJobCreatePrintOptions(
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.cupsColorSpace=%u (%s)", options->header.cupsColorSpace, cups_cspace_string(options->header.cupsColorSpace));
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.cupsNumColors=%u", options->header.cupsNumColors);
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.HWResolution=[%u %u]", options->header.HWResolution[0], options->header.HWResolution[1]);
-
+  papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "header.PWG_ImageBox=[%u %u %u %u]", options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxLeft], options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxTop], options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxRight], options->header.cupsInteger[CUPS_RASTER_PWG_ImageBoxBottom]);
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "num_pages=%u", options->num_pages);
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "copies=%d", options->copies);
   papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "finishings=0x%x", options->finishings);
@@ -590,6 +583,9 @@ _papplJobProcessRaster(
   unsigned		page = 0,	// Current page
 			x,		// Current column
 			y;		// Current line
+  int			job_pages_per_set;
+					// "job-pages-per-set" value, if any
+  unsigned		next_copy;	// Next copy boundary
 
 
   // Start processing the job...
@@ -601,7 +597,7 @@ _papplJobProcessRaster(
   // Open the raster stream...
   if ((ras = cupsRasterOpenIO((cups_raster_cb_t)httpRead, client->http, CUPS_RASTER_READ)) == NULL)
   {
-    papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to open raster stream from client - %s", cupsLastErrorString());
+    papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to open raster stream from client - %s", cupsGetErrorString());
     job->state = IPP_JSTATE_ABORTED;
     goto complete_job;
   }
@@ -609,12 +605,26 @@ _papplJobProcessRaster(
   // Prepare options...
   if (!cupsRasterReadHeader(ras, &header))
   {
-    papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to read raster stream from client - %s", cupsLastErrorString());
+    papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to read raster stream from client - %s", cupsGetErrorString());
     job->state = IPP_JSTATE_ABORTED;
     goto complete_job;
   }
 
-  if ((header_pages = header.cupsInteger[CUPS_RASTER_PWG_TotalPageCount]) > 0)
+  if ((job_pages_per_set = ippGetInteger(ippFindAttribute(job->attrs, "job-pages-per-set", IPP_TAG_INTEGER), 0)) > 0)
+  {
+    // Use the job-pages-per-set value to set the number of impressions...
+    papplJobSetImpressions(job, job_pages_per_set);
+
+    // Track copies at page boundaries...
+    next_copy = (unsigned)job_pages_per_set;
+  }
+  else
+  {
+    // Don't track copies...
+    next_copy = 0;
+  }
+
+  if ((header_pages = header.cupsInteger[CUPS_RASTER_PWG_TotalPageCount]) > 0 && job_pages_per_set == 0)
     papplJobSetImpressions(job, (int)header.cupsInteger[CUPS_RASTER_PWG_TotalPageCount]);
 
   options = papplJobCreatePrintOptions(job, (unsigned)job->impressions, header.cupsBitsPerPixel > 8);
@@ -631,10 +641,9 @@ _papplJobProcessRaster(
     if (job->is_canceled)
       break;
 
-    page ++;
     papplJobSetImpressionsCompleted(job, 1);
 
-    papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Page %u raster data is %ux%ux%u (%s)", page, header.cupsWidth, header.cupsHeight, header.cupsBitsPerPixel, cups_cspace_string(header.cupsColorSpace));
+    papplLogJob(job, PAPPL_LOGLEVEL_INFO, "Page %u raster data is %ux%ux%u (%s)", page + 1, header.cupsWidth, header.cupsHeight, header.cupsBitsPerPixel, cups_cspace_string(header.cupsColorSpace));
 
     papplSystemAddEvent(printer->system, printer, job, PAPPL_EVENT_JOB_PROGRESS, NULL);
 
@@ -810,20 +819,37 @@ _papplJobProcessRaster(
       break;
     }
 
+    page ++;
+
+    if (page == next_copy)
+    {
+      // Report a completed copy...
+      papplJobSetCopiesCompleted(job, 1);
+      next_copy += (unsigned)job_pages_per_set;
+    }
+
     if (job->is_canceled)
+    {
       break;
+    }
     else if (y < header.cupsHeight)
     {
-      papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to read page from raster stream from client - %s", cupsLastErrorString());
+      papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to read page from raster stream from client - %s", cupsGetErrorString());
       job->state = IPP_JSTATE_ABORTED;
       break;
     }
   }
   while (cupsRasterReadHeader(ras, &header));
 
+  if (next_copy == 0)
+  {
+    // Not tracking copies so record this as a single completed copy...
+    papplJobSetCopiesCompleted(job, 1);
+  }
+
   if (!(printer->driver_data.rendjob_cb)(job, options, job->printer->device))
     job->state = IPP_JSTATE_ABORTED;
-  else if (header_pages == 0)
+  else if (header_pages == 0 && job_pages_per_set == 0)
     papplJobSetImpressions(job, (int)page);
 
   complete_job:
@@ -846,6 +872,60 @@ _papplJobProcessRaster(
 }
 
 
+//
+// 'papplJobResume()' - Resume processing of a job.
+//
+
+void
+papplJobResume(pappl_job_t     *job,	// I - Job
+              pappl_jreason_t remove)	// I - Reasons to remove from "job-state-reasons"
+{
+  // Range check input...
+  if (!job)
+    return;
+
+  // Update state...
+  _papplRWLockWrite(job);
+
+  if (job->state == IPP_JSTATE_STOPPED)
+  {
+    job->state         = IPP_JSTATE_PENDING;
+    job->state_reasons &= ~remove;
+  }
+
+  _papplRWUnlock(job);
+
+  _papplRWLockWrite(job->printer);
+  _papplPrinterCheckJobsNoLock(job->printer);
+  _papplRWUnlock(job->printer);
+}
+
+
+//
+// 'papplJobSuspend()' - Temporarily stop processing of a job.
+//
+
+void
+papplJobSuspend(pappl_job_t     *job,	// I - Job
+               pappl_jreason_t add)	// I - Reasons to add to "job-state-reasons"
+{
+  // Range check input...
+  if (!job)
+    return;
+
+  // Update state...
+  _papplRWLockWrite(job);
+
+  if (job->state < IPP_JSTATE_STOPPED)
+  {
+    job->state         = IPP_JSTATE_STOPPED;
+    job->state_reasons |= add;
+  }
+
+  _papplRWUnlock(job);
+}
+
+
 //
 // 'cups_cspace_string()' - Get a string corresponding to a cupsColorSpace enum value.
 //
@@ -965,6 +1045,16 @@ finish_job(pappl_job_t  *job)		// I - Job
 {
   pappl_printer_t *printer = job->printer;
 					// Printer
+  static const char * const job_states[] =
+  {
+    "Pending",
+    "Held",
+    "Processing",
+    "Stopped",
+    "Canceled",
+    "Aborted",
+    "Completed"
+  };
 
 
   _papplRWLockWrite(printer);
@@ -975,20 +1065,20 @@ finish_job(pappl_job_t  *job)		// I - Job
   else if (job->state == IPP_JSTATE_PROCESSING)
     job->state = IPP_JSTATE_COMPLETED;
 
-  papplLogJob(job, PAPPL_LOGLEVEL_INFO, "%s, job-impressions-completed=%d.", job->state == IPP_JSTATE_COMPLETED ? "Completed" : job->state == IPP_JSTATE_CANCELED ? "Canceled" : "Aborted", job->impcompleted);
+  papplLogJob(job, PAPPL_LOGLEVEL_INFO, "%s, job-impressions-completed=%d.", job_states[job->state - IPP_JSTATE_PENDING], job->impcompleted);
 
   if (job->state >= IPP_JSTATE_CANCELED)
     job->completed = time(NULL);
 
+  _papplJobSetRetain(job);
+
   printer->processing_job = NULL;
 
-  if (!printer->max_preserved_jobs)
+  if (job->state >= IPP_JSTATE_CANCELED && !printer->max_preserved_jobs && !job->retain_until)
     _papplJobRemoveFile(job);
 
   _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_COMPLETED, NULL);
 
-  _papplRWUnlock(job);
-
   if (printer->is_stopped)
   {
     // New printer-state is 'stopped'...
@@ -1011,6 +1101,8 @@ finish_job(pappl_job_t  *job)		// I - Job
   if (!job->system->clean_time)
     job->system->clean_time = time(NULL) + 60;
 
+  _papplRWUnlock(job);
+
   _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, NULL);
 
   if (printer->max_preserved_jobs > 0)
@@ -1020,9 +1112,10 @@ finish_job(pappl_job_t  *job)		// I - Job
 
   _papplSystemConfigChanged(printer->system);
 
-  if (printer->is_deleted)
+  if (papplPrinterIsDeleted(printer))
   {
     papplPrinterDelete(printer);
+    return;
   }
   else if (!strncmp(printer->device_uri, "file:", 5) || cupsArrayGetCount(printer->active_jobs) == 0)
   {
@@ -1034,7 +1127,7 @@ finish_job(pappl_job_t  *job)		// I - Job
     papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device read metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.read_requests, (unsigned long)metrics.read_bytes, (unsigned long)metrics.read_msecs);
     papplLogJob(job, PAPPL_LOGLEVEL_DEBUG, "Device write metrics: %lu requests, %lu bytes, %lu msecs", (unsigned long)metrics.write_requests, (unsigned long)metrics.write_bytes, (unsigned long)metrics.write_msecs);
 
-    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device for job %d.", job->job_id);
+//    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device for job %d.", job->job_id);
 
     papplDeviceClose(printer->device);
     printer->device = NULL;
@@ -1042,8 +1135,12 @@ finish_job(pappl_job_t  *job)		// I - Job
     _papplRWUnlock(printer);
   }
 
-  if (cupsArrayGetCount(printer->active_jobs) > 0)
-    _papplPrinterCheckJobs(printer);
+  if (papplPrinterGetNumberOfActiveJobs(printer) > 0)
+  {
+    _papplRWLockWrite(printer);
+    _papplPrinterCheckJobsNoLock(printer);
+    _papplRWUnlock(printer);
+  }
 }
 
 
@@ -1054,9 +1151,10 @@ finish_job(pappl_job_t  *job)		// I - Job
 static bool				// O - `true` on success, `false` otherwise
 start_job(pappl_job_t *job)		// I - Job
 {
+  bool		ret = false;		// Return value
   pappl_printer_t *printer = job->printer;
 					// Printer
-  bool	first_open = true;		// Is this the first time we try to open the device?
+  bool		first_open = true;	// Is this the first time we try to open the device?
 
 
   // Move the job to the 'processing' state...
@@ -1088,7 +1186,7 @@ start_job(pappl_job_t *job)		// I - Job
 
   while (!printer->device && !printer->is_deleted && !job->is_canceled && papplSystemIsRunning(printer->system))
   {
-    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for job %d.", job->job_id);
+//    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for job %d.", job->job_id);
 
     printer->device = papplDeviceOpen(printer->device_uri, job->name, papplLogDevice, job->system);
 
@@ -1121,6 +1219,12 @@ start_job(pappl_job_t *job)		// I - Job
     _papplRWLockRead(job);
     _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_STATE_CHANGED, NULL);
     _papplRWUnlock(job);
+
+    if (printer->device)
+    {
+      papplDeviceClose(printer->device);
+      printer->device = NULL;
+    }
   }
 
   if (printer->device)
@@ -1128,11 +1232,12 @@ start_job(pappl_job_t *job)		// I - Job
     // Move the printer to the 'processing' state...
     printer->state      = IPP_PSTATE_PROCESSING;
     printer->state_time = time(NULL);
+    ret                 = true;
   }
 
   _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, NULL);
 
   _papplRWUnlock(printer);
 
-  return (printer->device != NULL);
+  return (ret);
 }
