@@ -1,17 +1,13 @@
 //
 // Job object for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -105,6 +101,7 @@ _papplJobCreate(
   job->state   = IPP_JSTATE_HELD;
   job->system  = printer->system;
   job->created = time(NULL);
+  job->copies  = 1;
 
   if (attrs)
   {
@@ -120,6 +117,9 @@ _papplJobCreate(
 
     _papplCopyAttributes(job->attrs, attrs, NULL, IPP_TAG_JOB, 0);
 
+    if ((attr = ippFindAttribute(job->attrs, "copies", IPP_TAG_INTEGER)) != NULL)
+      job->copies = ippGetInteger(attr, 0);
+
     hold_until      = ippGetString(ippFindAttribute(attrs, "job-hold-until", IPP_TAG_KEYWORD), 0, NULL);
     hold_until_time = ippDateToTime(ippGetDate(ippFindAttribute(attrs, "job-hold-until-time", IPP_TAG_DATE), 0));
 
@@ -137,7 +137,9 @@ _papplJobCreate(
     }
   }
   else
+  {
     ippAddString(job->attrs, IPP_TAG_JOB, IPP_TAG_NAME, "job-name", NULL, job_name);
+  }
 
   if ((attr = ippAddString(job->attrs, IPP_TAG_JOB, IPP_TAG_NAME, "job-originating-user-name", NULL, username)) != NULL)
     job->username = ippGetString(attr, 0, NULL);
@@ -454,22 +456,10 @@ papplJobOpenFile(
   if (!directory)
     directory = job->system->directory;
 
-  if (access(directory, X_OK))
+  if (mkdir(directory, 0777) && errno != EEXIST)
   {
-    if (errno == ENOENT)
-    {
-      // Spool directory does not exist, might have been deleted...
-      if (mkdir(directory, 0777))
-      {
-        papplLogJob(job, PAPPL_LOGLEVEL_FATAL, "Unable to create spool directory '%s': %s", directory, strerror(errno));
-        return (-1);
-      }
-    }
-    else
-    {
-      papplLogJob(job, PAPPL_LOGLEVEL_FATAL, "Unable to access spool directory '%s': %s", directory, strerror(errno));
-      return (-1);
-    }
+    papplLogJob(job, PAPPL_LOGLEVEL_FATAL, "Unable to create spool directory '%s': %s", directory, strerror(errno));
+    return (-1);
   }
 
   // Make a name from the job-name attribute...
@@ -547,7 +537,7 @@ papplJobRelease(pappl_job_t *job,	// I - Job
     return (false);
 
   // Lock the job and printer...
-  _papplRWLockRead(job->printer);
+  _papplRWLockWrite(job->printer);
   _papplRWLockWrite(job);
 
   // Only release jobs in the held state...
@@ -560,9 +550,10 @@ papplJobRelease(pappl_job_t *job,	// I - Job
 
   // Unlock and return...
   _papplRWUnlock(job);
-  _papplRWUnlock(job->printer);
 
-  _papplPrinterCheckJobs(job->printer);
+  _papplPrinterCheckJobsNoLock(job->printer);
+
+  _papplRWUnlock(job->printer);
 
   return (ret);
 }
@@ -622,6 +613,177 @@ _papplJobRemoveFile(pappl_job_t *job)	// I - Job
 }
 
 
+//
+// 'papplJobRetain()' - Retain a completed job until the specified time.
+//
+
+bool					// O - `true` on success, `false` on failure
+papplJobRetain(
+    pappl_job_t *job,			// I - Job
+    const char  *username,		// I - User that held the job or `NULL` for none/system
+    const char  *until,			// I - "job-retain-until" value or `NULL` for none
+    int         until_interval,		// I - "job-retain-until-interval" value or `0` for none
+    time_t      until_time)		// I - "job-retain-until-time" value or `0` for none
+{
+  bool	ret = false;			// Return value
+
+
+  // Range check input
+  if (!job)
+    return (false);
+
+  // Lock the printer and job so we can change it...
+  _papplRWLockRead(job->printer);
+  _papplRWLockWrite(job);
+
+  // Only hold jobs that haven't entered the processing state...
+  if (job->state < IPP_JSTATE_CANCELED)
+  {
+    // Hold until the specified time...
+    ret = _papplJobRetainNoLock(job, username, until, until_interval, until_time);
+  }
+
+  _papplRWUnlock(job);
+  _papplRWUnlock(job->printer);
+
+  return (ret);
+}
+
+
+//
+// '_papplJobRetainNoLock()' - Retain a completed job until the specified time.
+//
+
+bool					// O - `true` on success, `false` on failure
+_papplJobRetainNoLock(
+    pappl_job_t *job,			// I - Job
+    const char  *username,		// I - User that held the job or `NULL` for none/system
+    const char  *until,			// I - "job-retain-until" value or `NULL` for none
+    int         until_interval,		// I - "job-retain-until-interval" value or `0` for none
+    time_t      until_time)		// I - "job-retain-until-time" value or `0` for none
+{
+  ipp_attribute_t	*attr;		// "job-retain-until[-interval,-time]" attribute
+
+
+  // Update attributes...
+  if ((attr = ippFindAttribute(job->attrs, "job-retain-until", IPP_TAG_KEYWORD)) != NULL)
+  {
+    if (until)
+      ippSetString(job->attrs, &attr, 0, until);
+    else
+      ippDeleteAttribute(job->attrs, attr);
+  }
+  else if (until)
+  {
+    ippAddString(job->attrs, IPP_TAG_JOB, IPP_TAG_KEYWORD, "job-retain-until", NULL, until);
+  }
+
+  if ((attr = ippFindAttribute(job->attrs, "job-retain-until-interval", IPP_TAG_INTEGER)) != NULL)
+  {
+    if (until_interval > 0)
+      ippSetInteger(job->attrs, &attr, 0, until_interval);
+    else
+      ippDeleteAttribute(job->attrs, attr);
+  }
+  else if (until_interval > 0)
+  {
+    ippAddInteger(job->attrs, IPP_TAG_JOB, IPP_TAG_INTEGER, "job-retain-until-interval", until_interval);
+  }
+
+  if ((attr = ippFindAttribute(job->attrs, "job-retain-until-time", IPP_TAG_DATE)) != NULL)
+  {
+    if (until_time > 0)
+      ippSetDate(job->attrs, &attr, 0, ippTimeToDate(until_time));
+    else
+      ippDeleteAttribute(job->attrs, attr);
+  }
+  else if (until_time > 0)
+  {
+    ippAddDate(job->attrs, IPP_TAG_JOB, "job-retain-until-time", ippTimeToDate(until_time));
+  }
+
+  if (username)
+  {
+    _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_CONFIG_CHANGED, "Job retain set by '%s'.", username);
+  }
+
+  return (true);
+}
+
+
+//
+// '_papplJobSetRetain()' - Set the "retain_until" value for a Job.
+//
+
+void
+_papplJobSetRetain(pappl_job_t *job)	// I - Job
+{
+  ipp_attribute_t	*attr;		// "job-retain-until[-interval,-time]" attribute
+
+
+  if ((attr = ippFindAttribute(job->attrs, "job-retain-until", IPP_TAG_KEYWORD)) != NULL)
+  {
+    // Hold until the specified time period...
+    const char	*until = ippGetString(attr, 0, NULL);
+					// Keyword value
+    time_t	curtime;		// Current time
+    struct tm	curdate;		// Current date
+
+    time(&curtime);
+    localtime_r(&curtime, &curdate);
+
+    if (!strcmp(until, "day-time"))
+    {
+      // Hold to 6am the next morning unless local time is < 6pm.
+      if (curdate.tm_hour < 18)
+	job->retain_until = curtime;
+      else
+	job->retain_until = curtime + ((29 - curdate.tm_hour) * 60 + 59 - curdate.tm_min) * 60 + 60 - curdate.tm_sec;
+    }
+    else if (!strcmp(until, "evening") || !strcmp(until, "night"))
+    {
+      // Hold to 6pm unless local time is > 6pm or < 6am.
+      if (curdate.tm_hour < 6 || curdate.tm_hour >= 18)
+	job->retain_until = curtime;
+      else
+	job->retain_until = curtime + ((17 - curdate.tm_hour) * 60 + 59 - curdate.tm_min) * 60 + 60 - curdate.tm_sec;
+    }
+    else if (!strcmp(until, "second-shift"))
+    {
+      // Hold to 4pm unless local time is > 4pm.
+      if (curdate.tm_hour >= 16)
+	job->retain_until = curtime;
+      else
+	job->retain_until = curtime + ((15 - curdate.tm_hour) * 60 + 59 - curdate.tm_min) * 60 + 60 - curdate.tm_sec;
+    }
+    else if (!strcmp(until, "third-shift"))
+    {
+      // Hold to 12am unless local time is < 8am.
+      if (curdate.tm_hour < 8)
+	job->retain_until = curtime;
+      else
+	job->retain_until = curtime + ((23 - curdate.tm_hour) * 60 + 59 - curdate.tm_min) * 60 + 60 - curdate.tm_sec;
+    }
+    else if (!strcmp(until, "weekend"))
+    {
+      // Hold to weekend unless we are in the weekend.
+      if (curdate.tm_wday == 0 || curdate.tm_wday == 6)
+	job->retain_until = curtime;
+      else
+	job->retain_until = curtime + (((5 - curdate.tm_wday) * 24 + (17 - curdate.tm_hour)) * 60 + 59 - curdate.tm_min) * 60 + 60 - curdate.tm_sec;
+    }
+  }
+  else if ((attr = ippFindAttribute(job->attrs, "job-retain-until-interval", IPP_TAG_INTEGER)) != NULL)
+  {
+    job->retain_until = time(NULL) + ippGetInteger(attr, 0);
+  }
+  else if ((attr = ippFindAttribute(job->attrs, "job-retain-until-time", IPP_TAG_DATE)) != NULL)
+  {
+    job->retain_until = ippDateToTime(ippGetDate(attr, 0));
+  }
+}
+
+
 //
 // '_papplJobSubmitFile()' - Submit a file for printing.
 //
@@ -631,6 +793,8 @@ _papplJobSubmitFile(
     pappl_job_t *job,			// I - Job
     const char  *filename)		// I - Filename
 {
+  _papplRWLockWrite(job);
+
   if (!job->format)
   {
     // Open the file
@@ -645,6 +809,8 @@ _papplJobSubmitFile(
       headersize = read(fd, (char *)header, sizeof(header));
       close(fd);
 
+      _papplRWLockRead(job->system);
+
       if (!memcmp(header, "%PDF", 4))
 	job->format = "application/pdf";
       else if (!memcmp(header, "%!", 2))
@@ -659,6 +825,8 @@ _papplJobSubmitFile(
 	job->format = "image/urf";
       else if (job->system->mime_cb)
 	job->format = (job->system->mime_cb)(header, (size_t)headersize, job->system->mime_cbdata);
+
+      _papplRWUnlock(job->system);
     }
   }
 
@@ -695,7 +863,18 @@ _papplJobSubmitFile(
     {
       // Process the job...
       job->state = IPP_JSTATE_PENDING;
-      _papplPrinterCheckJobs(job->printer);
+
+      _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_STATE_CHANGED, NULL);
+
+      _papplRWUnlock(job);
+      _papplRWLockWrite(job->printer);
+      _papplPrinterCheckJobsNoLock(job->printer);
+      _papplRWUnlock(job->printer);
+    }
+    else
+    {
+      // Queue the job...
+      _papplRWUnlock(job);
     }
   }
   else
@@ -707,6 +886,10 @@ _papplJobSubmitFile(
     job->state     = IPP_JSTATE_ABORTED;
     job->completed = time(NULL);
 
+    _papplSystemAddEventNoLock(job->system, job->printer, job, PAPPL_EVENT_JOB_COMPLETED, "Job aborted.");
+
+    _papplRWUnlock(job);
+
     papplLogJob(job, PAPPL_LOGLEVEL_ERROR, "Unable to allocate filename.");
 
     if (!strncmp(filename, job->system->directory, dirlen) && filename[dirlen] == '/')
@@ -724,11 +907,11 @@ _papplJobSubmitFile(
 
 
 //
-// '_papplPrinterCheckJobs()' - Check for new jobs to process.
+// '_papplPrinterCheckJobsNoLock()' - Check for new jobs to process.
 //
 
 void
-_papplPrinterCheckJobs(
+_papplPrinterCheckJobsNoLock(
     pappl_printer_t *printer)		// I - Printer
 {
   pappl_job_t	*job;			// Current job
@@ -757,8 +940,6 @@ _papplPrinterCheckJobs(
     return;
   }
 
-  _papplRWLockWrite(printer);
-
   // Enumerate the jobs.  Since we have a writer (exclusive) lock, we are the
   // only thread enumerating and can use cupsArrayGetFirst/Last...
   for (job = (pappl_job_t *)cupsArrayGetFirst(printer->active_jobs); job; job = (pappl_job_t *)cupsArrayGetNext(printer->active_jobs))
@@ -789,15 +970,15 @@ _papplPrinterCheckJobs(
 	  printer->system->clean_time = time(NULL) + 60;
       }
       else
+      {
 	pthread_detach(t);
+      }
       break;
     }
   }
 
   if (!job)
     papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "No jobs to process at this time.");
-
-  _papplRWUnlock(printer);
 }
 
 
@@ -830,13 +1011,16 @@ _papplPrinterCleanJobsNoLock(
     {
       if (job->filename)
       {
-	preserved ++;
-	if (preserved > printer->max_preserved_jobs)
+	if ((preserved + 1) > printer->max_preserved_jobs || (job->retain_until && time(NULL) > job->retain_until))
 	  _papplJobRemoveFile(job);
+	else
+	  preserved ++;
       }
     }
     else
+    {
       break;
+    }
   }
 }
 
