@@ -1,7 +1,7 @@
 //
 // Public job header file for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2023 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -44,7 +44,11 @@ enum pappl_jreason_e			// IPP "job-state-reasons" bit values
   PAPPL_JREASON_PROCESSING_TO_STOP_POINT = 0x00080000,	// 'processing-to-stop-point'
   PAPPL_JREASON_QUEUED_IN_DEVICE = 0x00100000,		// 'queued-in-device'
   PAPPL_JREASON_WARNINGS_DETECTED = 0x00200000,		// 'warnings-detected'
-  PAPPL_JREASON_JOB_HOLD_UNTIL_SPECIFIED = 0x00400000	// 'job-hold-until-specified'
+  PAPPL_JREASON_JOB_HOLD_UNTIL_SPECIFIED = 0x00400000,	// 'job-hold-until-specified'
+  PAPPL_JREASON_JOB_CANCELED_AFTER_TIMEOUT = 0x00800000,// 'job-canceled-after-timeout'
+  PAPPL_JREASON_JOB_FETCHABLE = 0x01000000,		// 'job-fetchable'
+  PAPPL_JREASON_JOB_SUSPENDED_FOR_APPROVAL = 0x02000000	// 'job-suspended-for-approval'
+
 };
 typedef unsigned int pappl_jreason_t;	// Bitfield for IPP "job-state-reasons" values
 
@@ -62,6 +66,8 @@ extern void		papplJobDeletePrintOptions(pappl_pr_options_t *options);
 extern bool		papplJobFilterImage(pappl_job_t *job, pappl_device_t *device, pappl_pr_options_t *options, const unsigned char *pixels, int width, int height, int depth, int ppi, bool smoothing) _PAPPL_PUBLIC;
 
 extern ipp_attribute_t	*papplJobGetAttribute(pappl_job_t *job, const char *name) _PAPPL_PUBLIC;
+extern int		papplJobGetCopies(pappl_job_t *job) _PAPPL_PUBLIC;
+extern int		papplJobGetCopiesCompleted(pappl_job_t *job) _PAPPL_PUBLIC;
 extern void		*papplJobGetData(pappl_job_t *job) _PAPPL_PUBLIC;
 extern const char	*papplJobGetFilename(pappl_job_t *job) _PAPPL_PUBLIC;
 extern const char	*papplJobGetFormat(pappl_job_t *job) _PAPPL_PUBLIC;
@@ -85,12 +91,16 @@ extern bool		papplJobIsCanceled(pappl_job_t *job) _PAPPL_PUBLIC;
 extern int		papplJobOpenFile(pappl_job_t *job, char *fname, size_t fnamesize, const char *directory, const char *ext, const char *mode) _PAPPL_PUBLIC;
 
 extern bool		papplJobRelease(pappl_job_t *job, const char *username) _PAPPL_PUBLIC;
+extern void		papplJobResume(pappl_job_t *job, pappl_jreason_t remove) _PAPPL_PUBLIC;
+extern bool		papplJobRetain(pappl_job_t *job, const char *username, const char *until, int until_interval, time_t until_time) _PAPPL_PUBLIC;
 
+extern void		papplJobSetCopiesCompleted(pappl_job_t *job, int add) _PAPPL_PUBLIC;
 extern void		papplJobSetData(pappl_job_t *job, void *data) _PAPPL_PUBLIC;
 extern void		papplJobSetImpressions(pappl_job_t *job, int impressions) _PAPPL_PUBLIC;
 extern void		papplJobSetImpressionsCompleted(pappl_job_t *job, int add) _PAPPL_PUBLIC;
 extern void		papplJobSetMessage(pappl_job_t *job, const char *message, ...) _PAPPL_PUBLIC _PAPPL_FORMAT(2,3);
 extern void		papplJobSetReasons(pappl_job_t *job, pappl_jreason_t add, pappl_jreason_t remove) _PAPPL_PUBLIC;
+extern void		papplJobSuspend(pappl_job_t *job, pappl_jreason_t add) _PAPPL_PUBLIC;
 
 
 #  ifdef __cplusplus
