@@ -1,16 +1,12 @@
 //
 // Logging functions for the Printer Application Framework
 //
-// Copyright © 2019-2021 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "client-private.h"
 #include "job-private.h"
 #include "log-private.h"
@@ -26,16 +22,14 @@
 // Local functions...
 //
 
-static void	rotate_log(pappl_system_t *system);
-static void	write_log(pappl_system_t *system, pappl_loglevel_t level, const char *message, va_list ap);
+static void	rotate_log_no_lock(pappl_system_t *system);
+static void	write_log_no_lock(pappl_system_t *system, pappl_loglevel_t level, const char *message, va_list ap);
 
 
 //
 // Local globals...
 //
 
-static pthread_mutex_t	log_mutex = PTHREAD_MUTEX_INITIALIZER;
-					// Log rotation mutex
 #if !_WIN32
 static const int	syslevels[] =	// Mapping of log levels to syslog
 {
@@ -90,17 +84,23 @@ papplLog(pappl_system_t   *system,	// I - System
     return;
   }
 
-  if (level < system->loglevel)
+  if (level < papplSystemGetLogLevel(system))
     return;
 
   va_start(ap, message);
 
-  if (system->logfd >= 0)
-    write_log(system, level, message, ap);
 #if !_WIN32
-  else
+  if (system->log_is_syslog)
+  {
     vsyslog(syslevels[level], message, ap);
+  }
+  else
 #endif // !_WIN32
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    write_log_no_lock(system, level, message, ap);
+    pthread_mutex_unlock(&system->log_mutex);
+  }
 
   va_end(ap);
 }
@@ -109,7 +109,7 @@ papplLog(pappl_system_t   *system,	// I - System
 //
 // '_papplLogAttributes()' - Log IPP attributes for a client connection.
 //
-// This function logs the IPP attributes sent or recieved on a client
+// This function logs the IPP attributes sent or received on a client
 // connection at the `PAPPL_LOGLEVEL_DEBUG` (debug) log level.
 //
 
@@ -132,7 +132,7 @@ _papplLogAttributes(
   if (!client || !title || !ipp)
     return;
 
-  if (client->system->loglevel > PAPPL_LOGLEVEL_DEBUG)
+  if (papplSystemGetLogLevel(client->system) > PAPPL_LOGLEVEL_DEBUG)
     return;
 
   major = ippGetVersion(ipp, &minor);
@@ -187,23 +187,32 @@ papplLogClient(
 {
   char		cmessage[1024];		// Message with client prefix
   va_list	ap;			// Pointer to arguments
+  pappl_system_t *system;		// System
 
 
   if (!client || !message)
     return;
 
-  if (level < client->system->loglevel)
+  system = client->system;
+
+  if (level < papplSystemGetLogLevel(system))
     return;
 
   snprintf(cmessage, sizeof(cmessage), "[Client %d] %s", client->number, message);
   va_start(ap, message);
 
-  if (client->system->logfd >= 0)
-    write_log(client->system, level, cmessage, ap);
 #if !_WIN32
-  else
+  if (system->log_is_syslog)
+  {
     vsyslog(syslevels[level], cmessage, ap);
+  }
+  else
 #endif // !_WIN32
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    write_log_no_lock(system, level, cmessage, ap);
+    pthread_mutex_unlock(&system->log_mutex);
+  }
 
   va_end(ap);
 }
@@ -254,58 +263,67 @@ papplLogJob(
 {
   char		jmessage[1024];		// Message with job prefix
   va_list	ap;			// Pointer to arguments
+  pappl_system_t *system;		// System
 
 
   if (!job || !message)
     return;
 
-  if (level < job->system->loglevel)
+  system = job->system;
+
+  if (level < papplSystemGetLogLevel(system))
     return;
 
   snprintf(jmessage, sizeof(jmessage), "[Job %d] %s", job->job_id, message);
   va_start(ap, message);
 
-  if (job->system->logfd >= 0)
-    write_log(job->system, level, jmessage, ap);
 #if !_WIN32
-  else
+  if (system->log_is_syslog)
+  {
     vsyslog(syslevels[level], jmessage, ap);
+  }
+  else
 #endif // !_WIN32
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    write_log_no_lock(system, level, jmessage, ap);
+    pthread_mutex_unlock(&system->log_mutex);
+  }
 
   va_end(ap);
 }
 
 
 //
-// '_papplLogOpen()' - Open the log file
+// '_papplLogOpenNoLock()' - Open the log file
 //
 
 void
-_papplLogOpen(
+_papplLogOpenNoLock(
     pappl_system_t *system)		// I - System
 {
   // Open the log file...
-  if (!strcmp(system->logfile, "syslog"))
+  if (system->log_is_syslog)
   {
     // Log to syslog...
-    system->logfd = -1;
+    system->log_fd = -1;
   }
-  else if (!strcmp(system->logfile, "-"))
+  else if (!strcmp(system->log_file, "-"))
   {
     // Log to stderr...
-    system->logfd = 2;
+    system->log_fd = 2;
   }
   else
   {
-    int	oldfd = system->logfd;		// Old log file descriptor
+    int	oldfd = system->log_fd;		// Old log file descriptor
 
     // Log to a file...
-    if ((system->logfd = open(system->logfile, O_CREAT | O_WRONLY | O_APPEND | O_NOFOLLOW | O_CLOEXEC, 0600)) < 0)
+    if ((system->log_fd = open(system->log_file, O_CREAT | O_WRONLY | O_APPEND | O_NOFOLLOW | O_CLOEXEC, 0600)) < 0)
     {
       // Fallback to logging to stderr if we can't open the log file...
-      perror(system->logfile);
+      perror(system->log_file);
 
-      system->logfd = 2;
+      system->log_fd = 2;
     }
 
     // Close any old file...
@@ -314,7 +332,23 @@ _papplLogOpen(
   }
 
   // Log the system status information
-  papplLog(system, PAPPL_LOGLEVEL_INFO, "Starting log, system up %ld second(s), %d printer(s), listening for connections on '%s:%d' from up to %d clients.", (long)(time(NULL) - system->start_time), (int)cupsArrayGetCount(system->printers), system->hostname, system->port, system->max_clients);
+  if (system->log_level <= PAPPL_LOGLEVEL_INFO && !system->log_is_syslog)
+  {
+    struct timeval curtime;		// Current time
+    struct tm	curdate;		// Current date
+    char	message[1024];		// First log message
+
+    gettimeofday(&curtime, NULL);
+#if _WIN32
+    time_t curtemp = (time_t)curtime.tv_sec;
+    gmtime_s(&curdate, &curtemp);
+#else
+    gmtime_r(&curtime.tv_sec, &curdate);
+#endif // _WIN32
+
+    snprintf(message, sizeof(message), "I [%04d-%02d-%02dT%02d:%02d:%02d.%03dZ] Starting log, system up %ld second(s), %u printer(s), listening for connections on '%s:%d' from up to %u clients.\n", curdate.tm_year + 1900, curdate.tm_mon + 1, curdate.tm_mday, curdate.tm_hour, curdate.tm_min, curdate.tm_sec, (int)(curtime.tv_usec / 1000), (long)(time(NULL) - system->start_time), (unsigned)cupsArrayGetCount(system->printers), system->hostname, system->port, (unsigned)system->max_clients);
+    write(system->log_fd, message, strlen(message));
+  }
 }
 
 
@@ -346,12 +380,15 @@ papplLogPrinter(
 		*pptr,			// Pointer into prefix
 		*nameptr;		// Pointer into printer name
   va_list	ap;			// Pointer to arguments
+  pappl_system_t *system;		// System
 
 
   if (!printer || !message)
     return;
 
-  if (level < printer->system->loglevel)
+  system = printer->system;
+
+  if (level < papplSystemGetLogLevel(system))
     return;
 
   // Prefix the message with "[Printer foo]", making sure to not insert any
@@ -370,57 +407,65 @@ papplLogPrinter(
   // Write the log message...
   va_start(ap, message);
 
-  if (printer->system->logfd >= 0)
-    write_log(printer->system, level, pmessage, ap);
 #if !_WIN32
-  else
+  if (system->log_is_syslog)
+  {
     vsyslog(syslevels[level], pmessage, ap);
+  }
+  else
 #endif // !_WIN32
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    write_log_no_lock(system, level, pmessage, ap);
+    pthread_mutex_unlock(&system->log_mutex);
+  }
 
   va_end(ap);
 }
 
 
 //
-// 'rotate_log()' - Rotate the log file...
+// 'rotate_log_no_lock()' - Rotate the log file...
 //
 
 static void
-rotate_log(pappl_system_t *system)	// I - System
+rotate_log_no_lock(
+    pappl_system_t *system)		// I - System
 {
   struct stat	loginfo;		// Lof file information
 
 
   // Re-check whether we need to rotate the log file...
-  if (!fstat(system->logfd, &loginfo) && loginfo.st_size >= (off_t)system->logmaxsize)
+  if (!fstat(system->log_fd, &loginfo) && loginfo.st_size >= (off_t)system->log_max_size)
   {
     // Rename existing log file to "xxx.O"
     char	backname[1024];		// Backup log filename
 
 #if _WIN32
     // Windows doesn't allow an open file to be renamed...
-    close(system->logfd);
-    system->logfd = -1;
+    close(system->log_fd);
+    system->log_fd = -1;
 #endif // _WIN32
 
-    snprintf(backname, sizeof(backname), "%s.O", system->logfile);
+    snprintf(backname, sizeof(backname), "%s.O", system->log_file);
     unlink(backname);
-    rename(system->logfile, backname);
+    rename(system->log_file, backname);
 
-    _papplLogOpen(system);
+    _papplLogOpenNoLock(system);
   }
 }
 
 
 //
-// 'write_log()' - Write a line to the log file...
+// 'write_log_no_lock()' - Write a line to the log file...
 //
 
 static void
-write_log(pappl_system_t   *system,	// I - System
-          pappl_loglevel_t level,	// I - Log level
-          const char       *message,	// I - Printf-style message string
-          va_list          ap)		// I - Pointer to additional arguments
+write_log_no_lock(
+    pappl_system_t   *system,		// I - System
+    pappl_loglevel_t level,		// I - Log level
+    const char       *message,		// I - Printf-style message string
+    va_list          ap)		// I - Pointer to additional arguments
 {
   struct stat	loginfo;		// Log file information
   char		buffer[2048],		// Output buffer
@@ -428,7 +473,8 @@ write_log(pappl_system_t   *system,	// I - System
 		*bufend;		// Pointer to end of buffer
   struct timeval curtime;		// Current time
   struct tm	curdate;		// Current date
-  static const char *prefix = "DIWEF";	// Message prefix
+  char		prefix;			// Message prefix
+  static const char *prefixes = "DIWEF";// Message prefixes
   const char	*sval;			// String value
   char		size,			// Size character (h, l, L)
 		type;			// Format type character
@@ -438,14 +484,6 @@ write_log(pappl_system_t   *system,	// I - System
 		*tptr;			// Pointer into temporary format
 
 
-  // Rotate log as needed...
-  if (system->logmaxsize > 0 && !fstat(system->logfd, &loginfo) && loginfo.st_size >= (off_t)system->logmaxsize)
-  {
-    pthread_mutex_lock(&log_mutex);
-    rotate_log(system);
-    pthread_mutex_unlock(&log_mutex);
-  }
-
   // Each log line starts with a standard prefix of log level and date/time...
   gettimeofday(&curtime, NULL);
 #if _WIN32
@@ -455,14 +493,21 @@ write_log(pappl_system_t   *system,	// I - System
   gmtime_r(&curtime.tv_sec, &curdate);
 #endif // _WIN32
 
-  snprintf(buffer, sizeof(buffer), "%c [%04d-%02d-%02dT%02d:%02d:%02d.%03dZ] ", prefix[level], curdate.tm_year + 1900, curdate.tm_mon + 1, curdate.tm_mday, curdate.tm_hour, curdate.tm_min, curdate.tm_sec, (int)(curtime.tv_usec / 1000));
+  if (level < PAPPL_LOGLEVEL_DEBUG)
+    prefix = 'd';
+  else if (level > PAPPL_LOGLEVEL_FATAL)
+    prefix = 'f';
+  else
+    prefix = prefixes[level];
+
+  snprintf(buffer, sizeof(buffer), "%c [%04d-%02d-%02dT%02d:%02d:%02d.%03dZ] ", prefix, curdate.tm_year + 1900, curdate.tm_mon + 1, curdate.tm_mday, curdate.tm_hour, curdate.tm_min, curdate.tm_sec, (int)(curtime.tv_usec / 1000));
   bufptr = buffer + 29;			// Skip level/date/time
   bufend = buffer + sizeof(buffer) - 1;	// Leave room for newline on end
 
   // Then format the message line using printf format sequences...
   while (*message && bufptr < bufend)
   {
-    if (*message == '%')
+    if (*message == '%' && message[1])
     {
       tptr    = tformat;
       *tptr++ = *message++;
@@ -565,7 +610,7 @@ write_log(pappl_system_t   *system,	// I - System
 	case 'e' :
 	case 'f' :
 	case 'g' :
-	    snprintf(bufptr, (size_t)(bufptr - bufend + 1), tformat, va_arg(ap, double));
+	    snprintf(bufptr, (size_t)(bufend - bufptr), tformat, va_arg(ap, double));
 	    bufptr += strlen(bufptr);
 	    break;
 
@@ -579,18 +624,18 @@ write_log(pappl_system_t   *system,	// I - System
 	case 'x' :
 #  ifdef HAVE_LONG_LONG
             if (size == 'L')
-	      snprintf(bufptr, (size_t)(bufptr - bufend + 1), tformat, va_arg(ap, long long));
+	      snprintf(bufptr, (size_t)(bufend - bufptr), tformat, va_arg(ap, long long));
 	    else
 #  endif // HAVE_LONG_LONG
             if (size == 'l')
-	      snprintf(bufptr, (size_t)(bufptr - bufend + 1), tformat, va_arg(ap, long));
+	      snprintf(bufptr, (size_t)(bufend - bufptr), tformat, va_arg(ap, long));
 	    else
-	      snprintf(bufptr, (size_t)(bufptr - bufend + 1), tformat, va_arg(ap, int));
+	      snprintf(bufptr, (size_t)(bufend - bufptr), tformat, va_arg(ap, int));
             bufptr += strlen(bufptr);
             break;
 
         case 'p' : // Log a pointer
-            snprintf(bufptr, (size_t)(bufptr - bufend + 1), "%p", va_arg(ap, void *));
+            snprintf(bufptr, (size_t)(bufend - bufptr), "%p", va_arg(ap, void *));
             bufptr += strlen(bufptr);
             break;
 
@@ -651,7 +696,7 @@ write_log(pappl_system_t   *system,	// I - System
             break;
 
         default : // Something else we don't support
-            papplCopyString(bufptr, tformat, (size_t)(bufptr - bufend + 1));
+            papplCopyString(bufptr, tformat, (size_t)(bufend - bufptr));
             bufptr += strlen(bufptr);
             break;
       }
@@ -660,8 +705,13 @@ write_log(pappl_system_t   *system,	// I - System
       *bufptr++ = *message++;
   }
 
-  // Add a newline and write it out...
+  // Add a newline...
   *bufptr++ = '\n';
 
-  write(system->logfd, buffer, (size_t)(bufptr - buffer));
+  // Rotate log as needed...
+  if (system->log_max_size > 0 && !fstat(system->log_fd, &loginfo) && loginfo.st_size >= (off_t)system->log_max_size)
+    rotate_log_no_lock(system);
+
+  // Write the log entry...
+  write(system->log_fd, buffer, (size_t)(bufptr - buffer));
 }
