@@ -1,7 +1,7 @@
 //
 // Private header file for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2023 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -10,7 +10,7 @@
 #ifndef _PAPPL_PAPPL_PRIVATE_H_
 #  define _PAPPL_PAPPL_PRIVATE_H_
 #  include "client-private.h"
-#  include "device.h"
+#  include "device-private.h"
 #  include "dnssd-private.h"
 #  include "job-private.h"
 #  include "loc-private.h"
