@@ -1,16 +1,12 @@
 //
 // Printer accessor functions for the Printer Application Framework
 //
-// Copyright © 2020-2022 by Michael R Sweet.
+// Copyright © 2020-2024 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "printer-private.h"
 #include "job-private.h"
 #include "system-private.h"
@@ -27,28 +23,27 @@ void
 papplPrinterCloseDevice(
     pappl_printer_t *printer)		// I - Printer
 {
-  if (!printer || !printer->device || !printer->device_in_use)
+  if (!printer)
     return;
 
-  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Done using device for status/maintenance.");
-
-  printer->device_in_use = false;
-
-  if (cupsArrayGetCount(printer->active_jobs) > 0 && !printer->processing_job)
-    _papplPrinterCheckJobs(printer);
-
-  if (printer->state != IPP_PSTATE_PROCESSING)
+  _papplRWLockWrite(printer);
+  if (printer->device && printer->device_in_use)
   {
-    _papplRWLockWrite(printer);
+    printer->device_in_use = false;
 
-    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device.");
+    if (cupsArrayGetCount(printer->active_jobs) > 0 && !printer->processing_job)
+      _papplPrinterCheckJobsNoLock(printer);
 
-    papplDeviceClose(printer->device);
+    if (printer->state != IPP_PSTATE_PROCESSING)
+    {
+//      papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Closing device.");
 
-    printer->device = NULL;
-
-    _papplRWUnlock(printer);
+      papplDeviceClose(printer->device);
+      printer->device = NULL;
+    }
   }
+
+  _papplRWUnlock(printer);
 }
 
 
@@ -522,17 +517,30 @@ pappl_preason_t				// O - "printer-state-reasons" bit values
 papplPrinterGetReasons(
     pappl_printer_t *printer)		// I - Printer
 {
-  if (!printer)
-    return (PAPPL_PREASON_NONE);
+  pappl_preason_t ret = PAPPL_PREASON_NONE;
+					// Return value
 
-  if (!printer->device_in_use && !printer->processing_job && (time(NULL) - printer->status_time) > 1 && printer->driver_data.status_cb)
+
+  if (printer)
   {
-    // Update printer status...
-    (printer->driver_data.status_cb)(printer);
-    printer->status_time = time(NULL);
+    _papplRWLockRead(printer);
+
+    if (!printer->device_in_use && !printer->processing_job && (time(NULL) - printer->status_time) > 1 && printer->driver_data.status_cb)
+    {
+      // Update printer status...
+      _papplRWUnlock(printer);
+      (printer->driver_data.status_cb)(printer);
+
+      _papplRWLockRead(printer);
+      printer->status_time = time(NULL);
+    }
+
+    ret = printer->state_reasons;
+
+    _papplRWUnlock(printer);
   }
 
-  return (printer->state_reasons);
+  return (ret);
 }
 
 
@@ -894,14 +902,14 @@ papplPrinterOpenDevice(
   pappl_device_t	*device = NULL;	// Open device
 
 
-  if (!printer || printer->device_in_use || printer->processing_job || !printer->device_uri)
+  if (!printer)
     return (NULL);
 
   _papplRWLockWrite(printer);
 
-  if (!printer->device_in_use && !printer->processing_job)
+  if (!printer->device_in_use && !printer->processing_job && printer->device_uri)
   {
-    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for status/maintenance.");
+//    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Opening device for status/maintenance.");
 
     printer->device        = device = papplDeviceOpen(printer->device_uri, "printer", papplLogDevice, printer->system);
     printer->device_in_use = device != NULL;
@@ -987,10 +995,10 @@ papplPrinterReleaseHeldNewJobs(
     }
   }
 
-  _papplRWUnlock(printer);
-
   if (released_jobs)
-    _papplPrinterCheckJobs(printer);
+    _papplPrinterCheckJobsNoLock(printer);
+
+  _papplRWUnlock(printer);
 
   return (ret);
 }
@@ -1016,9 +1024,9 @@ papplPrinterResume(
 
   _papplSystemAddEventNoLock(printer->system, printer, NULL, PAPPL_EVENT_PRINTER_STATE_CHANGED, "Resumed printer.");
 
-  _papplRWUnlock(printer);
+  _papplPrinterCheckJobsNoLock(printer);
 
-  _papplPrinterCheckJobs(printer);
+  _papplRWUnlock(printer);
 }
 
 
@@ -1100,13 +1108,13 @@ papplPrinterSetGeoLocation(
     return;
 
   // Validate geo-location - must be NULL or a "geo:" URI...
-  if (value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
+  if (value && *value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
     return;
 
   _papplRWLockWrite(printer);
 
   free(printer->geo_location);
-  printer->geo_location = value ? strdup(value) : NULL;
+  printer->geo_location = value && *value ? strdup(value) : NULL;
   printer->config_time  = time(NULL);
 
   _papplPrinterRegisterDNSSDNoLock(printer);
