@@ -1,16 +1,12 @@
 //
 // Printer driver functions for the Printer Application Framework
 //
-// Copyright © 2020-2022 by Michael R Sweet.
+// Copyright © 2020-2024 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "printer-private.h"
 #include "system-private.h"
 
@@ -237,21 +233,22 @@ papplPrinterSetDriverDefaults(
   _papplRWLockWrite(printer);
 
   // Copy xxx_default values...
+  printer->driver_data.bin_default            = data->bin_default;
   printer->driver_data.color_default          = data->color_default;
   printer->driver_data.content_default        = data->content_default;
+  printer->driver_data.darkness_configured    = data->darkness_configured;
+  printer->driver_data.darkness_default       = data->darkness_default;
+  printer->driver_data.identify_default       = data->identify_default;
+  printer->driver_data.media_default          = data->media_default;
+  printer->driver_data.mode_configured        = data->mode_configured;
+  printer->driver_data.orient_default         = data->orient_default;
   printer->driver_data.quality_default        = data->quality_default;
   printer->driver_data.scaling_default        = data->scaling_default;
   printer->driver_data.sides_default          = data->sides_default;
+  printer->driver_data.speed_default          = data->speed_default;
+  printer->driver_data.tear_offset_configured = data->tear_offset_configured;
   printer->driver_data.x_default              = data->x_default;
   printer->driver_data.y_default              = data->y_default;
-  printer->driver_data.media_default          = data->media_default;
-  printer->driver_data.speed_default          = data->speed_default;
-  printer->driver_data.darkness_default       = data->darkness_default;
-  printer->driver_data.bin_default            = data->bin_default;
-  printer->driver_data.mode_configured        = data->mode_configured;
-  printer->driver_data.tear_offset_configured = data->tear_offset_configured;
-  printer->driver_data.darkness_configured    = data->darkness_configured;
-  printer->driver_data.identify_default       = data->identify_default;
 
   // Copy any vendor-specific xxx-default values...
   for (i = 0; i < data->num_vendor; i ++)
@@ -389,18 +386,6 @@ make_attrs(
   char			output_tray[256];// "printer-output-tray" value
   _pappl_mime_filter_t	*filter;	// Current filter
   ipp_attribute_t	*attr;		// Attribute
-  static const int	fnvalues[] =	// "finishings" values
-  {
-    IPP_FINISHINGS_PUNCH,
-    IPP_FINISHINGS_STAPLE,
-    IPP_FINISHINGS_TRIM
-  };
-  static const char * const fnstrings[] =
-  {					// "finishing-template" values
-    "punch",
-    "staple",
-    "trim"
-  };
   static const char * const job_creation_attributes[] =
   {					// job-creation-attributes-supported values
     "copies",
@@ -411,16 +396,17 @@ make_attrs(
     "job-hold-until-time",
     "job-name",
     "job-priority",
+    "job-retain-until",
+    "job-retain-until-interval",
+    "job-retain-until-time",
     "media",
     "media-col",
     "multiple-document-handling",
     "orientation-requested",
-    "output-bin",
     "print-color-mode",
     "print-content-optimize",
     "print-quality",
-    "printer-resolution",
-    "sides"
+    "printer-resolution"
   };
   static const char * const media_col[] =
   {					// media-col-supported values
@@ -443,7 +429,6 @@ make_attrs(
   static const char * const printer_settable_attributes[] =
   {					// printer-settable-attributes values
     "copies-default",
-    "document-format-default",
     "media-col-default",
     "media-col-ready",
     "media-default",
@@ -453,6 +438,7 @@ make_attrs(
     "print-color-mode-default",
     "print-content-optimize-default",
     "print-quality-default",
+    "printer-contact-col",
     "printer-geo-location",
     "printer-location",
     "printer-organization",
@@ -521,11 +507,11 @@ make_attrs(
     if (data->finishings & bit)
     {
       cvalues[num_values   ] = ippNew();
-      ippAddString(cvalues[num_values], IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "finishing-template", NULL, fnstrings[i]);
-      ivalues[num_values   ] = fnvalues[i];
-      svalues[num_values ++] = fnstrings[i];
+      ippAddString(cvalues[num_values], IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "finishing-template", NULL, _papplFinishingsString(bit));
+      ivalues[num_values   ] = (int)_papplFinishingsEnum(bit);
+      svalues[num_values ++] = _papplFinishingsString(bit);
 
-      snprintf(ptr, sizeof(fn) - (size_t)(ptr - fn), "-%d", fnvalues[i]);
+      snprintf(ptr, sizeof(fn) - (size_t)(ptr - fn), "-%d", (int)_papplFinishingsEnum(bit));
       ptr += strlen(ptr);
     }
   }
@@ -580,6 +566,15 @@ make_attrs(
   memcpy((void *)svalues, job_creation_attributes, sizeof(job_creation_attributes));
   num_values = (int)(sizeof(job_creation_attributes) / sizeof(job_creation_attributes[0]));
 
+  if (data->finishings)
+  {
+    svalues[num_values ++] = "finishings";
+    svalues[num_values ++] = "finishings-col";
+  }
+
+  if (data->num_bin)
+    svalues[num_values ++] = "output-bin";
+
   if (pdf_supported)
     svalues[num_values ++] = "page-ranges";
 
@@ -589,6 +584,9 @@ make_attrs(
   if (data->speed_supported[1])
     svalues[num_values ++] = "print-speed";
 
+  if (data->sides_supported != PAPPL_SIDES_ONE_SIDED)
+    svalues[num_values ++] = "sides";
+
   for (i = 0; i < (cups_len_t)data->num_vendor && i < (cups_len_t)(sizeof(data->vendor) / sizeof(data->vendor[0])); i ++)
     svalues[num_values ++] = data->vendor[i];
 
@@ -957,6 +955,77 @@ make_attrs(
     ippAddInteger(attrs, IPP_TAG_PRINTER, IPP_TAG_INTEGER, "printer-darkness-supported", data->darkness_supported);
 
 
+  // printer-device-id
+  if (printer->device_id)
+  {
+    ippAddString(attrs, IPP_TAG_PRINTER, IPP_TAG_TEXT, "printer-device-id", NULL, printer->device_id);
+  }
+  else
+  {
+    // Generate printer-device-id value as needed...
+    char	mfg[128],		// Manufacturer name
+		*mdl,			// Model name
+		cmd[128];		// Command (format) list
+    ipp_attribute_t *formats;		// "document-format-supported" attribute
+    cups_len_t	count;			// Number of values
+
+    // Assume make and model are separated by a space...
+    papplCopyString(mfg, data->make_and_model, sizeof(mfg));
+    if ((mdl = strchr(mfg, ' ')) != NULL)
+      *mdl++ = '\0';			// Nul-terminate the make
+    else
+      mdl = mfg;			// No separator, so assume the make and model are the same
+
+    formats = ippFindAttribute(attrs, "document-format-supported", IPP_TAG_MIMETYPE);
+    count   = ippGetCount(formats);
+    for (i = 0, ptr = cmd; i < count; i ++)
+    {
+      const char *format = ippGetString(formats, i, NULL);
+					// Current MIME media type
+
+      if (!strcmp(format, "application/pdf"))
+        format = "PDF";
+      else if (!strcmp(format, "application/postscript"))
+        format = "PS";
+      else if (!strcmp(format, "application/vnd.eltron-epl"))
+        format = "EPL";
+      else if (!strcmp(format, "application/vnd.hp-pcl"))
+        format = "PCL";
+      else if (!strcmp(format, "application/vnd.sii-slp"))
+        format = "SIISLP";
+      else if (!strcmp(format, "application/vnd.tsc-tspl"))
+        format = "TSPL";
+      else if (!strcmp(format, "application/vnd.zebra-cpcl"))
+        format = "CPCL";
+      else if (!strcmp(format, "application/vnd.zebra-zpl"))
+        format = "ZPL";
+      else if (!strcmp(format, "image/jpeg"))
+        format = "JPEG";
+      else if (!strcmp(format, "image/png"))
+        format = "PNG";
+      else if (!strcmp(format, "image/pwg-raster"))
+        format = "PWGRaster";
+      else if (!strcmp(format, "image/urf"))
+        format = "URF";
+      else if (!strcmp(format, "text/plain"))
+        format = "TXT";
+      else if (!strcmp(format, "application/octet-stream"))
+        continue;
+
+      if (ptr > cmd)
+        snprintf(ptr, sizeof(cmd) - (size_t)(ptr - cmd), ",%s", format);
+      else
+        papplCopyString(cmd, format, sizeof(cmd));
+
+      ptr += strlen(ptr);
+    }
+
+    *ptr = '\0';
+
+    ippAddStringf(attrs, IPP_TAG_PRINTER, IPP_TAG_TEXT, "printer-device-id", NULL, "MFG:%s;MDL:%s;CMD:%s;", mfg, mdl, cmd);
+  }
+
+
   // printer-kind
   for (num_values = 0, bit = PAPPL_KIND_DISC; bit <= PAPPL_KIND_ROLL; bit *= 2)
   {
@@ -1005,14 +1074,27 @@ make_attrs(
   memcpy((void *)svalues, printer_settable_attributes, sizeof(printer_settable_attributes));
   num_values = (int)(sizeof(printer_settable_attributes) / sizeof(printer_settable_attributes[0]));
 
+  if (data->finishings)
+  {
+    svalues[num_values ++] = "finishings-col-default";
+    svalues[num_values ++] = "finishings-default";
+  }
+
   if (data->mode_supported)
     svalues[num_values ++] = "label-mode-configured";
+
   if (data->tear_offset_supported[1])
     svalues[num_values ++] = "label-tear-off-configured";
+
+  if (data->num_bin)
+    svalues[num_values ++] = "output-bin-default";
+
   if (data->darkness_supported)
     svalues[num_values ++] = "print-darkness-default";
+
   if (data->speed_supported[1])
     svalues[num_values ++] = "print-speed-default";
+
   if (data->darkness_supported)
     svalues[num_values ++] = "printer-darkness-configured";
   if (system->wifi_join_cb)
@@ -1020,6 +1102,8 @@ make_attrs(
     svalues[num_values ++] = "printer-wifi-password";
     svalues[num_values ++] = "printer-wifi-ssid";
   }
+  if (data->sides_supported != PAPPL_SIDES_ONE_SIDED)
+    svalues[num_values ++] = "sides-default";
   for (i = 0; i < (cups_len_t)data->num_vendor && num_values < (int)(sizeof(svalues) / sizeof(svalues[0])); i ++)
   {
     snprintf(vvalues[i], sizeof(vvalues[0]), "%s-default", data->vendor[i]);
