@@ -1,17 +1,13 @@
 //
 // Printer IPP processing for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -74,8 +70,8 @@ _papplPrinterCopyAttributesNoLock(
 					// URL scheme for resources
 
 
-  _papplCopyAttributes(client->response, printer->attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
-  _papplCopyAttributes(client->response, printer->driver_attrs, ra, IPP_TAG_ZERO, IPP_TAG_CUPS_CONST);
+  _papplCopyAttributes(client->response, printer->attrs, ra, IPP_TAG_ZERO, true);
+  _papplCopyAttributes(client->response, printer->driver_attrs, ra, IPP_TAG_ZERO, false);
   _papplPrinterCopyStateNoLock(printer, IPP_TAG_PRINTER, client->response, client, ra);
 
   if (!ra || cupsArrayFind(ra, "copies-supported"))
@@ -102,8 +98,38 @@ _papplPrinterCopyAttributesNoLock(
       ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "identify-actions-default", NULL, "none");
   }
 
+  if (printer->max_preserved_jobs > 0)
+  {
+    static const char * const job_retain_until[] =
+    {					// job-retain-until-supported values
+      "day-time",
+      "evening",
+      "indefinite",
+      "night",
+      "no-hold",
+      "second-shift",
+      "third-shift",
+      "weekend"
+    };
+
+    if (!ra || cupsArrayFind(ra, "job-retain-until-default"))
+      ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-retain-until-default", NULL, "none");
+
+    if (!ra || cupsArrayFind(ra, "job-retain-until-interval-default"))
+      ippAddOutOfBand(client->response, IPP_TAG_PRINTER, IPP_TAG_NOVALUE, "job-retain-until-interval-default");
+
+    if (!ra || cupsArrayFind(ra, "job-retain-until-interval-supported"))
+      ippAddRange(client->response, IPP_TAG_PRINTER, "job-retain-until-interval-supported", 0, 86400);
+
+    if (!ra || cupsArrayFind(ra, "job-retain-until-supported"))
+      ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-retain-until-supported", (cups_len_t)(sizeof(job_retain_until) / sizeof(job_retain_until[0])), NULL, job_retain_until);
+
+    if (!ra || cupsArrayFind(ra, "job-retain-until-time-supported"))
+      ippAddRange(client->response, IPP_TAG_PRINTER, "job-retain-until-time-supported", 0, 86400);
+  }
+
   if (!ra || cupsArrayFind(ra, "job-spooling-supported"))
-    ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-spooling-supported", NULL, printer->max_active_jobs == 1 ? "stream" : "spool");
+    ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "job-spooling-supported", NULL, (printer->max_active_jobs == 1 || (format && (!strcmp(format, "image/pwg-raster") || !strcmp(format, "image/urf")))) ? "stream" : "spool");
 
   if ((!ra || cupsArrayFind(ra, "label-mode-configured")) && data->mode_configured)
     ippAddString(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "label-mode-configured", NULL, _papplLabelModeString(data->mode_configured));
@@ -434,7 +460,7 @@ _papplPrinterCopyAttributesNoLock(
 //    _papplRWUnlock(printer->system);
 
     if (num_values > 0)
-      ippAddStrings(printer->attrs, IPP_TAG_PRINTER, IPP_TAG_LANGUAGE, "printer-strings-languages-supported", IPP_NUM_CAST num_values, NULL, svalues);
+      ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_TAG_LANGUAGE, "printer-strings-languages-supported", IPP_NUM_CAST num_values, NULL, svalues);
   }
 
   if (!ra || cupsArrayFind(ra, "printer-strings-uri"))
@@ -940,6 +966,7 @@ _papplPrinterSetAttributes(
     { "media-default",			IPP_TAG_KEYWORD,	1 },
     { "media-ready",			IPP_TAG_KEYWORD,	PAPPL_MAX_SOURCE },
     { "orientation-requested-default",	IPP_TAG_ENUM,		1 },
+    { "output-bin-default",		IPP_TAG_KEYWORD,	1 },
     { "print-color-mode-default",	IPP_TAG_KEYWORD,	1 },
     { "print-content-optimize-default",	IPP_TAG_KEYWORD,	1 },
     { "print-darkness-default",		IPP_TAG_INTEGER,	1 },
@@ -953,7 +980,8 @@ _papplPrinterSetAttributes(
     { "printer-organizational-unit",	IPP_TAG_TEXT,		1 },
     { "printer-resolution-default",	IPP_TAG_RESOLUTION,	1 },
     { "printer-wifi-password",		IPP_TAG_STRING,		1 },
-    { "printer-wifi-ssid",		IPP_TAG_NAME,		1 }
+    { "printer-wifi-ssid",		IPP_TAG_NAME,		1 },
+    { "sides-default",			IPP_TAG_KEYWORD,	1 }
   };
 
 
@@ -1089,6 +1117,25 @@ _papplPrinterSetAttributes(
       driver_data.orient_default = (ipp_orient_t)ippGetInteger(rattr, 0);
       do_defaults = true;
     }
+    else if (!strcmp(name, "output-bin-default"))
+    {
+      const char *keyword = ippGetString(rattr, 0, NULL);
+					// Keyword value
+
+      for (i = 0; i < driver_data.num_bin; i ++)
+      {
+        if (!strcmp(keyword, driver_data.bin[i]))
+        {
+          driver_data.bin_default = i;
+          break;
+        }
+      }
+
+      if (i >= driver_data.num_bin)
+        papplClientRespondIPP(client, IPP_STATUS_ERROR_ATTRIBUTES_OR_VALUES, "Unsupported \"output-bin-default\" value '%s'.", keyword);
+      else
+        do_defaults = true;
+    }
     else if (!strcmp(name, "print-color-mode-default"))
     {
       driver_data.color_default = _papplColorModeValue(ippGetString(rattr, 0, NULL));
@@ -1178,6 +1225,21 @@ _papplPrinterSetAttributes(
       papplCopyString(wifi_ssid, ippGetString(rattr, 0, NULL), sizeof(wifi_ssid));
       do_wifi = true;
     }
+    else if (!strcmp(name, "sides-default"))
+    {
+      pappl_sides_t sides_default = _papplSidesValue(ippGetString(rattr, 0, NULL));
+					// Sides value
+
+      if (!sides_default || !(driver_data.sides_supported & sides_default))
+      {
+        papplClientRespondIPP(client, IPP_STATUS_ERROR_ATTRIBUTES_OR_VALUES, "Unsupported \"sides-default\" value '%s'.", ippGetString(rattr, 0, NULL));
+      }
+      else
+      {
+        driver_data.sides_default = sides_default;
+        do_defaults               = true;
+      }
+    }
   }
 
   if (ippGetStatusCode(client->response) != IPP_STATUS_OK)
@@ -1564,10 +1626,20 @@ ipp_get_printer_attributes(
 					// Printer
 
 
+  _papplRWLockRead(printer->system);
+  _papplRWLockRead(printer);
+
   if (!printer->device_in_use && !printer->processing_job && (time(NULL) - printer->status_time) > 1 && printer->driver_data.status_cb)
   {
     // Update printer status...
+    _papplRWUnlock(printer);
+    _papplRWUnlock(printer->system);
+
     (printer->driver_data.status_cb)(printer);
+
+    _papplRWLockRead(printer->system);
+    _papplRWLockWrite(printer);
+
     printer->status_time = time(NULL);
   }
 
@@ -1576,8 +1648,6 @@ ipp_get_printer_attributes(
 
   papplClientRespondIPP(client, IPP_STATUS_OK, NULL);
 
-  _papplRWLockRead(printer->system);
-  _papplRWLockRead(printer);
   _papplPrinterCopyAttributesNoLock(printer, client, ra, ippGetString(ippFindAttribute(client->request, "document-format", IPP_TAG_MIMETYPE), 0, NULL));
   _papplRWUnlock(printer);
   _papplRWUnlock(printer->system);
@@ -1856,7 +1926,7 @@ valid_job_attributes(
     }
 
     if (ippGetBoolean(attr, 0))
-      valid = true;
+      exact = true;
   }
 
   if ((attr = ippFindAttribute(client->request, "copies", IPP_TAG_ZERO)) != NULL)
