@@ -1,17 +1,12 @@
 //
 // Raw printing support for the Printer Application Framework
 //
-// Copyright © 2019-2021 by Michael R Sweet.
-// Copyright © 2010-2019 by Apple Inc.
+// Copyright © 2019-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -37,7 +32,7 @@ _papplPrinterAddRawListeners(
   {
     if ((sock = httpAddrListen(&(addrlist->addr), port)) < 0)
     {
-      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create socket print listener for '*:%d': %s", port, cupsLastErrorString());
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create socket print listener for '*:%d': %s", port, cupsGetErrorString());
     }
     else
     {
@@ -53,7 +48,7 @@ _papplPrinterAddRawListeners(
   {
     if ((sock = httpAddrListen(&(addrlist->addr), port)) < 0)
     {
-      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create socket print listener for '*:%d': %s", port, cupsLastErrorString());
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create socket print listener for '*:%d': %s", port, cupsGetErrorString());
     }
     else
     {
@@ -80,34 +75,38 @@ void *					// O - Thread exit value
 _papplPrinterRunRaw(
     pappl_printer_t *printer)		// I - Printer
 {
-  int	i;				// Looping var
+  int		i;			// Looping var
+  int		count;			// Return value from poll()
 
 
-  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Running socket print thread with %d listeners.", printer->num_raw_listeners);
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Socket listener thread starting with %d listeners.", printer->num_raw_listeners);
 
   _papplRWLockWrite(printer);
   printer->raw_active = true;
   _papplRWUnlock(printer);
 
-  while (!papplPrinterIsDeleted(printer) && papplSystemIsRunning(printer->system))
+  while (!papplPrinterIsDeleted(printer) && !_papplSystemIsShutdownNoLock(printer->system))
   {
     // Don't accept connections if we can't accept a new job...
     _papplRWLockRead(printer);
-    while ((int)cupsArrayGetCount(printer->active_jobs) >= printer->max_active_jobs && !printer->is_deleted && papplSystemIsRunning(printer->system))
+    if (printer->max_active_jobs > 0)
     {
-      _papplRWUnlock(printer);
-      usleep(100000);
-      _papplRWLockRead(printer);
+      while ((int)cupsArrayGetCount(printer->active_jobs) >= printer->max_active_jobs && !printer->is_deleted && !_papplSystemIsShutdownNoLock(printer->system))
+      {
+	_papplRWUnlock(printer);
+	usleep(1000);
+	_papplRWLockRead(printer);
+      }
     }
     _papplRWUnlock(printer);
 
-    if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+    if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
       break;
 
     // Wait 1 second for new connections...
-    if ((i = poll(printer->raw_listeners, (nfds_t)printer->num_raw_listeners, 1000)) > 0)
+    if ((count = poll(printer->raw_listeners, (nfds_t)printer->num_raw_listeners, 250)) > 0)
     {
-      if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+      if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
 	break;
 
       // Got a new connection request, accept from the corresponding listener...
@@ -163,13 +162,13 @@ _papplPrinterRunRaw(
 
           for (;;)
           {
-	    if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+	    if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
 	    {
 	      bytes = -1;
 	      break;
 	    }
 
-            if ((bytes = poll(&sockp, 1, 1000)) < 0)
+            if ((bytes = poll(&sockp, 1, 1000)) <= 0)
 	    {
 	      if ((time(NULL) - activity) >= 60)
 	        break;
@@ -177,10 +176,10 @@ _papplPrinterRunRaw(
 	        continue;
 	    }
 
-            activity = time(NULL);
-
             if (sockp.revents & POLLIN)
             {
+	      activity = time(NULL);
+
               if ((bytes = recv(sock, buffer, sizeof(buffer), 0)) > 0)
                 write(job->fd, buffer, (size_t)bytes);
               else
@@ -191,6 +190,10 @@ _papplPrinterRunRaw(
               bytes = -1;
               break;
             }
+            else if (sockp.revents & POLLHUP)
+	    {
+	      break;
+	    }
           }
 
           close(sock);
@@ -206,7 +209,9 @@ _papplPrinterRunRaw(
 	  // Finish the job...
 	  job->state = IPP_JSTATE_PENDING;
 
-	  _papplPrinterCheckJobs(printer);
+	  _papplRWLockWrite(printer);
+	  _papplPrinterCheckJobsNoLock(printer);
+	  _papplRWUnlock(printer);
 	  continue;
 
 	  // Abort the job...
@@ -226,7 +231,7 @@ _papplPrinterRunRaw(
         }
       }
     }
-    else if (i < 0 && errno != EAGAIN)
+    else if (count < 0 && errno != EAGAIN)
       break;
   }
 
@@ -234,5 +239,7 @@ _papplPrinterRunRaw(
   printer->raw_active = false;
   _papplRWUnlock(printer);
 
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Socket listener thread exiting on shutdown/delete.");
+
   return (NULL);
 }
