@@ -1,17 +1,13 @@
 //
 // USB printer class support for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 #include <cups/dir.h>
 #ifdef __linux
@@ -115,23 +111,26 @@ _papplPrinterRunUSB(
   time_t	device_time = 0;	// Last time moving data...
 
 
-  while (!papplPrinterIsDeleted(printer) && papplSystemIsRunning(printer->system))
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "USB listener thread starting.");
+
+  while (!papplPrinterIsDeleted(printer) && !_papplSystemIsShutdownNoLock(printer->system))
   {
+    _papplRWLockWrite(printer);
     if (!enable_usb_printer(printer, ifaces))
     {
-      _papplRWLockWrite(printer);
       disable_usb_printer(printer, ifaces);
       _papplRWUnlock(printer);
+
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "USB listener thread exiting on unrecoverable gadget error.");
       return (NULL);
     }
 
-    _papplRWLockWrite(printer);
     printer->usb_active = true;
     _papplRWUnlock(printer);
 
     sleep(1);
 
-    while ((data[0].fd = open("/dev/g_printer0", O_RDWR | O_EXCL)) < 0 && !papplPrinterIsDeleted(printer) && papplSystemIsRunning(printer->system))
+    while ((data[0].fd = open("/dev/g_printer0", O_RDWR | O_EXCL)) < 0 && !papplPrinterIsDeleted(printer) && !_papplSystemIsShutdownNoLock(printer->system))
     {
       if (errno != EBUSY && errno != ENODEV)
       {
@@ -155,18 +154,18 @@ _papplPrinterRunUSB(
 
     papplLogPrinter(printer, PAPPL_LOGLEVEL_INFO, "Monitoring USB for incoming print jobs.");
 
-    while (!papplPrinterIsDeleted(printer) && papplSystemIsRunning(printer->system))
+    while (!papplPrinterIsDeleted(printer) && !_papplSystemIsShutdownNoLock(printer->system))
     {
       if ((count = poll(data, NUM_IPP_USB + 1, 1000)) < 0)
       {
 	papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "USB poll failed: %s", strerror(errno));
 
-	if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+	if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
 	  break;
 
 	sleep(1);
       }
-      else if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+      else if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
       {
 	break;
       }
@@ -174,7 +173,12 @@ _papplPrinterRunUSB(
       {
 	papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "USB poll returned %d, revents=[%d %d %d %d].", count, data[0].revents, data[1].revents, data[2].revents, data[3].revents);
 
-	if (data[0].revents)
+        if (data[0].revents & (POLLERR | POLLHUP))
+        {
+	  // Hard error, need to close down the gadgets
+	  goto close_gadgets;
+	}
+	else if (data[0].revents)
 	{
 	  if (!device)
 	  {
@@ -186,7 +190,7 @@ _papplPrinterRunUSB(
 	      sleep(1);
 	    }
 
-	    if (papplPrinterIsDeleted(printer) || !papplSystemIsRunning(printer->system))
+	    if (papplPrinterIsDeleted(printer) || _papplSystemIsShutdownNoLock(printer->system))
 	    {
 	      papplPrinterCloseDevice(printer);
 	      break;
@@ -277,7 +281,12 @@ _papplPrinterRunUSB(
 	// Check for IPP-USB control messages...
 	for (i = 0; i < NUM_IPP_USB; i ++)
 	{
-	  if (data[i + 1].revents)
+          if (data[i + 1].revents & (POLLERR | POLLHUP))
+          {
+	    // Hard error, need to close down the gadgets
+	    goto close_gadgets;
+	  }
+	  else if (data[i + 1].revents)
 	  {
 	    struct usb_functionfs_event *event;
 					  // IPP-USB control event
@@ -363,6 +372,8 @@ _papplPrinterRunUSB(
   (void)printer;
 #endif // __linux
 
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "USB listener thread exiting on shutdown/delete.");
+
   return (NULL);
 }
 
@@ -396,15 +407,16 @@ papplPrinterSetUSB(
   if (printer)
   {
     // Don't allow changes once the gadget is running...
+    _papplRWLockWrite(printer);
+
     if (printer->usb_active)
     {
+      _papplRWUnlock(printer);
       papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "USB gadget options already set, unable to change.");
       return;
     }
 
     // Update the USB gadget settings...
-    _papplRWLockWrite(printer);
-
     printer->usb_vendor_id  = (unsigned short)vendor_id;
     printer->usb_product_id = (unsigned short)product_id;
     printer->usb_options    = options;
@@ -421,7 +433,7 @@ papplPrinterSetUSB(
     _papplRWUnlock(printer);
 
     // Start USB gadget if needed...
-    if (printer->system->is_running && printer->system->default_printer_id == printer->printer_id && (printer->system->options & PAPPL_SOPTIONS_USB_PRINTER))
+    if (papplSystemIsRunning(printer->system) && papplSystemGetDefaultPrinterID(printer->system) == printer->printer_id && (printer->system->options & PAPPL_SOPTIONS_USB_PRINTER))
     {
       pthread_t	tid;			// Thread ID
 
@@ -432,8 +444,17 @@ papplPrinterSetUSB(
       }
       else
       {
-	// Detach the main thread from the raw thread to prevent hangs...
+	// Detach the main thread from the USB thread to prevent hangs...
 	pthread_detach(tid);
+
+        _papplRWLockRead(printer);
+	while (!printer->usb_active)
+	{
+	  _papplRWUnlock(printer);
+	  usleep(1000);			// Wait for USB thread to start
+	  _papplRWLockRead(printer);
+	}
+	_papplRWUnlock(printer);
       }
     }
   }
@@ -456,7 +477,10 @@ create_directory(
     return (false);
   }
   else
+  {
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Created USB gadget directory '%s'.", filename);
     return (true);
+  }
 }
 
 
@@ -648,7 +672,7 @@ create_ipp_usb_iface(
     // Use UNIX domain socket...
     if ((iface->addrlist = httpAddrGetList(printer->system->domain_path, AF_LOCAL, "0")) == NULL)
     {
-      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to lookup '%s' for IPP USB gadget: %s", printer->system->domain_path, cupsLastErrorString());
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to lookup '%s' for IPP USB gadget: %s", printer->system->domain_path, cupsGetErrorString());
       return (false);
     }
   }
@@ -658,7 +682,7 @@ create_ipp_usb_iface(
     snprintf(filename, sizeof(filename), "%d", printer->system->port);
     if ((iface->addrlist = httpAddrGetList("localhost", AF_UNSPEC, filename)) == NULL)
     {
-      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to lookup 'localhost:%d' for IPP USB gadget: %s", printer->system->port, cupsLastErrorString());
+      papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to lookup 'localhost:%d' for IPP USB gadget: %s", printer->system->port, cupsGetErrorString());
       return (false);
     }
   }
@@ -701,11 +725,9 @@ create_string_file(
   else
     cupsFilePuts(fp, data);
 
-  if (cupsFileClose(fp))
-  {
-    papplLogPrinter(printer, PAPPL_LOGLEVEL_ERROR, "Unable to create USB gadget file '%s': %s", filename, strerror(errno));
-    return (false);
-  }
+  cupsFileClose(fp);
+
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Created USB gadget file '%s' containing '%s'.", filename, data);
 
   return (true);
 }
@@ -727,7 +749,10 @@ create_symlink(
     return (false);
   }
   else
+  {
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Created USB gadget symlink '%s' to '%s'.", filename, destname);
     return (true);
+  }
 }
 
 
@@ -814,6 +839,8 @@ disable_usb_printer(
     delete_ipp_usb_iface(ifaces + i);
 
   printer->usb_active = false;
+
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_INFO, "USB gadgets disabled.");
 }
 
 
@@ -953,6 +980,22 @@ enable_usb_printer(
   if (!create_directory(printer, gadget_dir))
     return (false);
 
+  snprintf(filename, sizeof(filename), "%s/bDeviceClass", gadget_dir);
+  if (!create_string_file(printer, filename, "0x00\n"))
+    return (false);
+
+  snprintf(filename, sizeof(filename), "%s/bDeviceSubClass", gadget_dir);
+  if (!create_string_file(printer, filename, "0x00\n"))
+    return (false);
+
+  snprintf(filename, sizeof(filename), "%s/bDeviceProtocol", gadget_dir);
+  if (!create_string_file(printer, filename, "0x00\n"))
+    return (false);
+
+  snprintf(filename, sizeof(filename), "%s/bcdUSB", gadget_dir);
+  if (!create_string_file(printer, filename, "0x0200\n"))
+    return (false);
+
   snprintf(filename, sizeof(filename), "%s/idVendor", gadget_dir);
   snprintf(temp, sizeof(temp), "0x%04x\n", printer->usb_vendor_id);
   if (!create_string_file(printer, filename, temp))
@@ -1092,6 +1135,8 @@ enable_usb_printer(
   {
     if (dent->filename[0] != '.' && dent->filename[0])
       break;
+
+    papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Ignoring UDC '%s'...", dent->filename);
   }
 
   if (!dent)
@@ -1119,6 +1164,8 @@ enable_usb_printer(
   // If we get here we need to tear down the IPP-USB interfaces...
   error:
 
+  papplLogPrinter(printer, PAPPL_LOGLEVEL_DEBUG, "Tearing down USB printer gadget.");
+
   for (i = 0; i < NUM_IPP_USB; i ++)
     delete_ipp_usb_iface(ifaces + i);
 
@@ -1146,7 +1193,7 @@ run_ipp_usb_iface(
 
   papplLogPrinter(iface->printer, PAPPL_LOGLEVEL_INFO, "IPP-USB%d: Starting.", iface->number);
 
-  while (!iface->printer->is_deleted && iface->printer->system->is_running)
+  while (!papplPrinterIsDeleted(iface->printer) && papplSystemIsRunning(papplPrinterGetSystem(iface->printer)))
   {
     // Wait for data from the host...
     if ((bytes = read(iface->ipp_to_device, hostbuf, sizeof(hostbuf))) > 0)
