@@ -1,17 +1,13 @@
 //
 // Printer web interface functions for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
+// Copyright © 2019-2024 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -24,7 +20,7 @@ static void	job_pager(pappl_client_t *client, pappl_printer_t *printer, int job_
 static char	*localize_keyword(pappl_client_t *client, const char *attrname, const char *keyword, char *buffer, size_t bufsize);
 static char	*localize_media(pappl_client_t *client, pappl_media_col_t *media, bool include_source, char *buffer, size_t bufsize);
 static void	media_chooser(pappl_client_t *client, pappl_pr_driver_data_t *driver_data, const char *title, const char *name, pappl_media_col_t *media);
-static char	*time_string(time_t tv, char *buffer, size_t bufsize);
+static char	*time_string(pappl_client_t *client, time_t tv, char *buffer, size_t bufsize);
 
 
 //
@@ -488,13 +484,16 @@ _papplPrinterWebDefaults(
   // print-darkness-configured
   if (data.darkness_supported)
   {
+    int darkness_value = ((data.darkness_supported - 1) * data.darkness_configured + 50) / 100;
+					// Scaled darkness value
+
     papplClientHTMLPrintf(client, "              <tr><th>%s:</th><td><select name=\"print-darkness\">", papplClientGetLocString(client, "print-darkness"));
     for (i = 0; i < data.darkness_supported; i ++)
     {
       int percent = 100 * i / (data.darkness_supported - 1);
 					// Percent darkness
 
-      papplClientHTMLPrintf(client, "<option value=\"%d\"%s>%d%%</option>", percent, percent == data.darkness_configured ? " selected" : "", percent);
+      papplClientHTMLPrintf(client, "<option value=\"%d\"%s>%d%%</option>", percent, i == darkness_value ? " selected" : "", percent);
     }
     papplClientHTMLPuts(client, "</select></td></tr>\n");
   }
@@ -682,7 +681,7 @@ _papplPrinterWebDelete(
     }
     else
     {
-      if (!printer->is_deleted)
+      if (!papplPrinterIsDeleted(printer))
       {
         papplPrinterDelete(printer);
         printer = NULL;
@@ -722,6 +721,12 @@ _papplPrinterWebHome(
   char		edit_path[1024];	// Edit configuration URL
   const int	limit = 20;		// Jobs per page
   int		job_index = 1;		// Job index
+  char		dns_sd_name[64],	// Printer DNS-SD name
+		location[128],		// Printer location
+		geo_location[128],	// Printer geo-location
+		organization[256],	// Printer organization
+		org_unit[256];		// Printer organizational unit
+  pappl_contact_t contact;		// Printer contact
 
 
   // Save current printer state...
@@ -858,7 +863,7 @@ _papplPrinterWebHome(
 
   _papplClientHTMLPutLinks(client, printer->links, PAPPL_LOPTIONS_CONFIGURATION);
 
-  _papplClientHTMLInfo(client, false, printer->dns_sd_name, printer->location, printer->geo_location, printer->organization, printer->org_unit, &printer->contact);
+  _papplClientHTMLInfo(client, false, papplPrinterGetDNSSDName(printer, dns_sd_name, sizeof(dns_sd_name)), papplPrinterGetLocation(printer, location, sizeof(location)), papplPrinterGetGeoLocation(printer, geo_location, sizeof(geo_location)), papplPrinterGetOrganization(printer, organization, sizeof(organization)), papplPrinterGetOrganizationalUnit(printer, org_unit, sizeof(org_unit)), papplPrinterGetContact(printer, &contact));
 
   if (!(printer->system->options & PAPPL_SOPTIONS_MULTI_QUEUE))
     _papplSystemWebSettings(client);
@@ -914,7 +919,6 @@ _papplPrinterWebIteratorCallback(
     pappl_printer_t *printer,		// I - Printer
     pappl_client_t  *client)		// I - Client
 {
-  int			i;		// Looping var
   pappl_preason_t	reason,		// Current reason
 			printer_reasons;// Printer state reasons
   ipp_pstate_t		printer_state;	// Printer state
@@ -942,11 +946,11 @@ _papplPrinterWebIteratorCallback(
 
   papplClientHTMLPrintf(client,
 			"          <p><img class=\"%s\" src=\"%s/icon-md.png\">%s, %s", ippEnumString("printer-state", (int)printer_state), printer->uriname, localize_keyword(client, "printer-state", state_str, text, sizeof(text)), jobs_str);
-  if ((printer->system->options & PAPPL_SOPTIONS_MULTI_QUEUE) && printer->printer_id == printer->system->default_printer_id)
+  if ((printer->system->options & PAPPL_SOPTIONS_MULTI_QUEUE) && printer->printer_id == papplSystemGetDefaultPrinterID(printer->system))
     papplClientHTMLPrintf(client, ", %s", papplClientGetLocString(client, _PAPPL_LOC("default printer")));
   if (printer->hold_new_jobs)
     papplClientHTMLPrintf(client, ", %s", papplClientGetLocString(client, _PAPPL_LOC("holding new jobs")));
-  for (i = 0, reason = PAPPL_PREASON_OTHER; reason <= PAPPL_PREASON_TONER_LOW; i ++, reason *= 2)
+  for (reason = PAPPL_PREASON_OTHER; reason <= PAPPL_PREASON_TONER_LOW; reason *= 2)
   {
     if (printer_reasons & reason)
       papplClientHTMLPrintf(client, ", %s", localize_keyword(client, "printer-state-reasons", _papplPrinterReasonString(reason), text, sizeof(text)));
@@ -1215,7 +1219,7 @@ _papplPrinterWebMedia(
   int			i;		// Looping var
   pappl_pr_driver_data_t data;		// Driver data
   char			name[32],	// Prefix (readyN)
-			text[256];	// Localized media-souce name
+			text[256];	// Localized media-source name
   const char		*status = NULL;	// Status message, if any
 
 
@@ -1455,13 +1459,13 @@ job_cb(pappl_job_t    *job,		// I - Job
     case IPP_JSTATE_PENDING :
 	show_cancel = true;
         show_hold   = papplPrinterGetMaxActiveJobs(papplJobGetPrinter(job)) != 1;
-	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued at %s"), time_string(papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued %s"), time_string(client, papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
 	break;
 
     case IPP_JSTATE_HELD :
 	show_cancel  = true;
 	show_release = true;
-	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued at %s"), time_string(papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Queued %s"), time_string(client, papplJobGetTimeCreated(job), hhmmss, sizeof(hhmmss)));
 	break;
 
     case IPP_JSTATE_PROCESSING :
@@ -1473,20 +1477,20 @@ job_cb(pappl_job_t    *job,		// I - Job
 	else
 	{
 	  show_cancel = true;
-	  papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Started at %s"), time_string(papplJobGetTimeProcessed(job), hhmmss, sizeof(hhmmss)));
+	  papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Started %s"), time_string(client, papplJobGetTimeProcessed(job), hhmmss, sizeof(hhmmss)));
 	}
 	break;
 
     case IPP_JSTATE_ABORTED :
-	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Aborted at %s"), time_string(papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Aborted %s"), time_string(client, papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
 	break;
 
     case IPP_JSTATE_CANCELED :
-	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Canceled at %s"), time_string(papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Canceled %s"), time_string(client, papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
 	break;
 
     case IPP_JSTATE_COMPLETED :
-	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Completed at %s"), time_string(papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
+	papplLocFormatString(papplClientGetLoc(client), when, sizeof(when), _PAPPL_LOC("Completed %s"), time_string(client, papplJobGetTimeCompleted(job), hhmmss, sizeof(hhmmss)));
 	break;
   }
 
@@ -1827,17 +1831,32 @@ media_chooser(
 // 'time_string()' - Return the local time in hours, minutes, and seconds.
 //
 
-static char *
-time_string(time_t tv,			// I - Time value
-            char   *buffer,		// I - Buffer
-	    size_t bufsize)		// I - Size of buffer
+static char *				// O - Formatted time string
+time_string(pappl_client_t *client,	// I - Client
+            time_t         tv,		// I - Time value
+            char           *buffer,	// I - Buffer
+	    size_t         bufsize)	// I - Size of buffer
 {
   struct tm	date;			// Local time and date
+  time_t	age;			// How old is the time?
 
+
+  // Get the local time in hours, minutes, and seconds...
   localtime_r(&tv, &date);
 
-  // TODO: Use locale-specific time format...
-  strftime(buffer, bufsize, "%X", &date);
+  // See how long ago this was...
+  age = time(NULL) - tv;
 
+  // Format based on the age...
+  if (age < 86400)
+    papplLocFormatString(papplClientGetLoc(client), buffer, bufsize, _PAPPL_LOC("at %02d:%02d:%02d"), date.tm_hour, date.tm_min, date.tm_sec);
+  else if (age < (2 * 86400))
+    papplLocFormatString(papplClientGetLoc(client), buffer, bufsize, _PAPPL_LOC("yesterday at %02d:%02d:%02d"), date.tm_hour, date.tm_min, date.tm_sec);
+  else if (age < (31 * 86400))
+    papplLocFormatString(papplClientGetLoc(client), buffer, bufsize, _PAPPL_LOC("%d days ago at %02d:%02d:%02d"), (int)(age / 86400), date.tm_hour, date.tm_min, date.tm_sec);
+  else
+    papplLocFormatString(papplClientGetLoc(client), buffer, bufsize, _PAPPL_LOC("%04d-%02d-%02d at %02d:%02d:%02d"), date.tm_year + 1900, date.tm_mon + 1, date.tm_mday, date.tm_hour, date.tm_min, date.tm_sec);
+
+  // Return the formatted string...
   return (buffer);
 }
