@@ -31,6 +31,8 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"  shutdown         Shutdown a running server.\" = \"  shutdown         Abschalten eines laufenden Servers.\";\n"
 "\"  status           Show server/printer/job status.\" = \"  status           Server/Drucker/Auftragsstatus anzeigen.\";\n"
 "\"  submit           Submit a file for printing.\" = \"  submit           Senden Sie eine Datei zum Drucken.\";\n"
+"\"%04d-%02d-%02d at %02d:%02d:%02d\" = \"%04d-%02d-%02d at %02d:%02d:%02d\";\n"
+"\"%d days ago at %02d:%02d:%02d\" = \"%d days ago at %02d:%02d:%02d\";\n"
 "\"%d inches/sec\" = \"%d zoll/s\";\n"
 "\"%d job\" = \"%d Job\";\n"
 /* Media size in millimeters */
@@ -72,13 +74,13 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Sub-command 'autoadd' is not supported.\" = \"%s: Unterbefehl 'autoadd' wird nicht unterstützt.\";\n"
 "\"%s: Too many files.\" = \"%s: Zu viele Dateien.\";\n"
 "\"%s: Unable to access '%s': %s\" = \"%s: Zugriff auf '%s': %s nicht möglich\";\n"
-"\"%s: Unable to add '%s': %s\" = \"%s: Hinzufügen von '%s': %s nicht möglich\";\n"
 "\"%s: Unable to add printer: %s\" = \"%s: Hinzufügen des Druckers nicht möglich: %s\";\n"
 "\"%s: Unable to cancel job: %s\" = \"%s: Job %s konnte nicht storniert werden\";\n"
 "\"%s: Unable to connect to printer at '%s:%d': %s\" = \"%s: Drucker konnte nicht verbunden werden: '%s:%d': %s\";\n"
 "\"%s: Unable to connect to server: %s\" = \"%s: Verbindung zu Server nicht möglich: %s\";\n"
 "\"%s: Unable to create temporary file: %s\" = \"%s: Temporäre Datei konnte nicht erstellt werden: %s\";\n"
 "\"%s: Unable to delete printer: %s\" = \"%s: Drucker konnte nicht gelöscht werden: %s\";\n"
+"\"%s: Unable to get available devices: %s\" = \"%s: Unable to get available devices: %s\";\n"
 "\"%s: Unable to get information for '%s': %s\" = \"%s: Information nicht verfügbar '%s': %s\";\n"
 "\"%s: Unable to get information for printer: %s\" = \"%s: Drucker-Information nicht verfügbar: %s\";\n"
 "\"%s: Unable to get printer options: %s\" = \"%s: Druckeroptionen konnten nicht abgerufen werden: %s\";\n"
@@ -97,7 +99,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Warning - user credentials are not supported in URIs.\" = \"%s: Warnung - Persönliche Benutzerinformationen werden nicht in URIs unterstützt.\";\n"
 "\", Borderless\" = \", Randlos\";\n"
 "\"A printer with that name already exists.\" = \"Ein Drucker mit diesem Namen existiert bereits.\";\n"
-"\"Aborted at %s\" = \"Abgebrochen bei %s\";\n"
+"\"Aborted %s\" = \"Aborted %s\";\n"
 "\"Add Printer\" = \"Drucker hinzufügen\";\n"
 "\"Admin Group\" = \"Admin Gruppe\";\n"
 "\"Afghanistan\" = \"Afghanistan\";\n"
@@ -149,7 +151,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Canada\" = \"Kanada\";\n"
 "\"Cancel All Jobs\" = \"Alle Druckaufträge löschen\";\n"
 "\"Cancel Job\" = \"Druckauftrag abbrechen\";\n"
-"\"Canceled at %s\" = \"Abgebrochen bei %s\";\n"
+"\"Canceled %s\" = \"Canceled %s\";\n"
 "\"Canceling\" = \"Abbrechend\";\n"
 "\"Cayman Islands\" = \"Cayman Inseln\";\n"
 "\"Central African Republic\" = \"Zentralafrikanische Republik\";\n"
@@ -166,10 +168,11 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chile\" = \"Chile\";\n"
 "\"China\" = \"China\";\n"
 "\"Christmas Island\" = \"Weihnachtsinsel\";\n"
+"\"City/Locality\" = \"City/Locality\";\n"
 "\"Cocos (Keeling) Islands\" = \"Kakao (Keeling) Inseln\";\n"
 "\"Colombia\" = \"Kolumbien\";\n"
 "\"Comoros\" = \"Komoren\";\n"
-"\"Completed at %s\" = \"Abgeschlossen bei %s\";\n"
+"\"Completed %s\" = \"Completed %s\";\n"
 "\"Configuration\" = \"Konfiguration\";\n"
 "\"Configure Media\" = \"Medien konfigurieren\";\n"
 "\"Confirm Cancel All\" = \"Abbruch bestätigen: Alle\";\n"
@@ -241,6 +244,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haiti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island und McDonald Inseln\";\n"
+"\"Hidden SSID\" = \"Hidden SSID\";\n"
 "\"Hold Job\" = \"Hold Job\";\n"
 "\"Hold New Jobs\" = \"Hold New Jobs\";\n"
 "\"Holding new jobs.\" = \"Holding new jobs.\";\n"
@@ -288,6 +292,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Job held.\" = \"Job held.\";\n"
 "\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Aufträge\";\n"
+"\"Joining Wi-Fi Network\" = \"Joining Wi-Fi Network\";\n"
 "\"Jordan\" = \"Jordanien\";\n"
 "\"Kazakhstan\" = \"Kasachstan\";\n"
 "\"Kenya\" = \"Kenia\";\n"
@@ -329,6 +334,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Micronesia (Federated States of)\" = \"Mikronesien (Bundesstaaten)\";\n"
 "\"Missing action.\" = \"Fehlende Aktivität.\";\n"
 "\"Missing job ID.\" = \"Missing job ID.\";\n"
+"\"Missing network name.\" = \"Missing network name.\";\n"
 "\"Missing private key.\" = \"Der private Schlüssel fehlt.\";\n"
 "\"Moldova, Republic of\" = \"Republik Moldau\";\n"
 "\"Monaco\" = \"Monaco\";\n"
@@ -403,13 +409,12 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Printing Defaults\" = \"Drucker Voreinstellungen\";\n"
 "\"Puerto Rico\" = \"Puerto Rico\";\n"
 "\"Qatar\" = \"Katar\";\n"
-"\"Queued at %s\" = \"In Warteschlange bei %s\";\n"
+"\"Queued %s\" = \"Queued %s\";\n"
 "\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
 "\"Release Job\" = \"Release Job\";\n"
 "\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Auftrag erneut drucken\";\n"
 "\"Reprinted job.\" = \"Reprinted job.\";\n"
-"\"Rescan\" = \"nochmals abtasten\";\n"
 "\"Resume Printing\" = \"Druck fortsetzen\";\n"
 "\"Reverse Landscape\" = \"Querformat (rotiert)\";\n"
 "\"Reverse Portrait\" = \"Hochformat (rotiert)\";\n"
@@ -452,8 +457,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"South Sudan\" = \"Südsudan\";\n"
 "\"Spain\" = \"Spanien\";\n"
 "\"Sri Lanka\" = \"Sri Lanka\";\n"
-"\"Started at %s\" = \"Gestartet bei %s\";\n"
-"\"State/Province\" = \"Staat/Provinz\";\n"
+"\"Started %s\" = \"Started %s\";\n"
 "\"Status\" = \"Status\";\n"
 "\"Sub-commands:\" = \"Unterbefehle:\";\n"
 "\"Sudan\" = \"Sudan\";\n"
@@ -489,9 +493,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to create certificate.\" = \"Zertifikat zu konnte nicht erstellt werden.\";\n"
 "\"Unable to create test print job.\" = \"Test-Druckauftrag konnte nicht erstellt werden.\";\n"
 "\"Unable to identify printer.\" = \"Drucker konnte nicht identifiziert werden.\";\n"
-"\"Unable to join Wi-Fi network.\" = \"Unfähig, mit Wi-Fi-Netzwerk zu verbinden.\";\n"
 "\"Unable to lookup address.\" = \"Adresse nicht gefunden.\";\n"
-"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Dieser Treiber konnte nicht benutzt werden.\";\n"
 "\"United Arab Emirates\" = \"Vereinigte Arabische Emirate\";\n"
 "\"United Kingdom\" = \"Vereinigtes Königreich\";\n"
@@ -502,6 +504,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unknown action.\" = \"Unbekannte Handlung.\";\n"
 "\"Unknown form action.\" = \"Unbekannte Formular Aktivität.\";\n"
 "\"Unknown form request.\" = \"Unbekannte Formularanfrage.\";\n"
+"\"Updating Network Configuration\" = \"Updating Network Configuration\";\n"
 "\"Uruguay\" = \"Uruguay\";\n"
 "\"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\" = \"Verwendung: %s SUB-COMMAND [OPTIONEN] (DATEINAME)\\n\";\n"
 "\"Use My Position\" = \"Verwenden Sie meine Position\";\n"
@@ -524,6 +527,7 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Yemen\" = \"Jemen\";\n"
 "\"Zambia\" = \"Sambia\";\n"
 "\"Zimbabwe\" = \"Simbabwe\";\n"
+"\"at %02d:%02d:%02d\" = \"at %02d:%02d:%02d\";\n"
 /* IPP attribute, keyword, and enum (integer) localizations */
 "\"copies\" = \"Anzahl der Kopien\";\n"
 "\"default printer\" = \"Standarddrucker\";\n"
@@ -1057,4 +1061,5 @@ static const char *de_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"sides.two-sided-long-edge\" = \"An (Hochformat)\";\n"
 "\"sides.two-sided-short-edge\" = \"An (Querformat)\";\n"
 "\"unable to join\" = \"Zusammenfügen fehlgeschlagen\";\n"
+"\"yesterday at %02d:%02d:%02d\" = \"yesterday at %02d:%02d:%02d\";\n"
 "\"Åland Islands\" = \"Åland (Inselgruppe)\";\n";
