@@ -31,6 +31,8 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"  shutdown         Shutdown a running server.\" = \"  shutdown         Shutdown a running server.\";\n"
 "\"  status           Show server/printer/job status.\" = \"  status           Show server/printer/job status.\";\n"
 "\"  submit           Submit a file for printing.\" = \"  submit           Submit a file for printing.\";\n"
+"\"%04d-%02d-%02d at %02d:%02d:%02d\" = \"%04d-%02d-%02d at %02d:%02d:%02d\";\n"
+"\"%d days ago at %02d:%02d:%02d\" = \"%d days ago at %02d:%02d:%02d\";\n"
 "\"%d inches/sec\" = \"%d inches/sec\";\n"
 "\"%d job\" = \"%d job\";\n"
 /* Media size in millimeters */
@@ -72,13 +74,13 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Sub-command 'autoadd' is not supported.\" = \"%s: Sub-command 'autoadd' is not supported.\";\n"
 "\"%s: Too many files.\" = \"%s: Too many files.\";\n"
 "\"%s: Unable to access '%s': %s\" = \"%s: Unable to access '%s': %s\";\n"
-"\"%s: Unable to add '%s': %s\" = \"%s: Unable to add '%s': %s\";\n"
 "\"%s: Unable to add printer: %s\" = \"%s: Unable to add printer: %s\";\n"
 "\"%s: Unable to cancel job: %s\" = \"%s: Unable to cancel job: %s\";\n"
 "\"%s: Unable to connect to printer at '%s:%d': %s\" = \"%s: Unable to connect to printer at '%s:%d': %s\";\n"
 "\"%s: Unable to connect to server: %s\" = \"%s: Unable to connect to server: %s\";\n"
 "\"%s: Unable to create temporary file: %s\" = \"%s: Unable to create temporary file: %s\";\n"
 "\"%s: Unable to delete printer: %s\" = \"%s: Unable to delete printer: %s\";\n"
+"\"%s: Unable to get available devices: %s\" = \"%s: Unable to get available devices: %s\";\n"
 "\"%s: Unable to get information for '%s': %s\" = \"%s: Unable to get information for '%s': %s\";\n"
 "\"%s: Unable to get information for printer: %s\" = \"%s: Unable to get information for printer: %s\";\n"
 "\"%s: Unable to get printer options: %s\" = \"%s: Unable to get printer options: %s\";\n"
@@ -97,7 +99,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Warning - user credentials are not supported in URIs.\" = \"%s: Warning - user credentials are not supported in URIs.\";\n"
 "\", Borderless\" = \", Borderless\";\n"
 "\"A printer with that name already exists.\" = \"A printer with that name already exists.\";\n"
-"\"Aborted at %s\" = \"Aborted at %s\";\n"
+"\"Aborted %s\" = \"Aborted %s\";\n"
 "\"Add Printer\" = \"Add Printer\";\n"
 "\"Admin Group\" = \"Admin Group\";\n"
 "\"Afghanistan\" = \"Afghanistan\";\n"
@@ -149,7 +151,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Canada\" = \"Canada\";\n"
 "\"Cancel All Jobs\" = \"Cancel All Jobs\";\n"
 "\"Cancel Job\" = \"Cancel Job\";\n"
-"\"Canceled at %s\" = \"Canceled at %s\";\n"
+"\"Canceled %s\" = \"Canceled %s\";\n"
 "\"Canceling\" = \"Canceling\";\n"
 "\"Cayman Islands\" = \"Cayman Islands\";\n"
 "\"Central African Republic\" = \"Central African Republic\";\n"
@@ -166,10 +168,11 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chile\" = \"Chile\";\n"
 "\"China\" = \"China\";\n"
 "\"Christmas Island\" = \"Christmas Island\";\n"
+"\"City/Locality\" = \"City/Locality\";\n"
 "\"Cocos (Keeling) Islands\" = \"Cocos (Keeling) Islands\";\n"
 "\"Colombia\" = \"Colombia\";\n"
 "\"Comoros\" = \"Comoros\";\n"
-"\"Completed at %s\" = \"Completed at %s\";\n"
+"\"Completed %s\" = \"Completed %s\";\n"
 "\"Configuration\" = \"Configuration\";\n"
 "\"Configure Media\" = \"Configure Media\";\n"
 "\"Confirm Cancel All\" = \"Confirm Cancel All\";\n"
@@ -241,6 +244,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haiti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island and McDonald Islands\";\n"
+"\"Hidden SSID\" = \"Hidden SSID\";\n"
 "\"Hold Job\" = \"Hold Job\";\n"
 "\"Hold New Jobs\" = \"Hold New Jobs\";\n"
 "\"Holding new jobs.\" = \"Holding new jobs.\";\n"
@@ -288,6 +292,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Job held.\" = \"Job held.\";\n"
 "\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"Jobs\";\n"
+"\"Joining Wi-Fi Network\" = \"Joining Wi-Fi Network\";\n"
 "\"Jordan\" = \"Jordan\";\n"
 "\"Kazakhstan\" = \"Kazakhstan\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -329,6 +334,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Micronesia (Federated States of)\" = \"Micronesia (Federated States of)\";\n"
 "\"Missing action.\" = \"Missing action.\";\n"
 "\"Missing job ID.\" = \"Missing job ID.\";\n"
+"\"Missing network name.\" = \"Missing network name.\";\n"
 "\"Missing private key.\" = \"Missing private key.\";\n"
 "\"Moldova, Republic of\" = \"Moldova, Republic of\";\n"
 "\"Monaco\" = \"Monaco\";\n"
@@ -403,13 +409,12 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Printing Defaults\" = \"Printing Defaults\";\n"
 "\"Puerto Rico\" = \"Puerto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
-"\"Queued at %s\" = \"Queued at %s\";\n"
+"\"Queued %s\" = \"Queued %s\";\n"
 "\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
 "\"Release Job\" = \"Release Job\";\n"
 "\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"Reprint Job\";\n"
 "\"Reprinted job.\" = \"Reprinted job.\";\n"
-"\"Rescan\" = \"Rescan\";\n"
 "\"Resume Printing\" = \"Resume Printing\";\n"
 "\"Reverse Landscape\" = \"Reverse Landscape\";\n"
 "\"Reverse Portrait\" = \"Reverse Portrait\";\n"
@@ -452,8 +457,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"South Sudan\" = \"South Sudan\";\n"
 "\"Spain\" = \"Spain\";\n"
 "\"Sri Lanka\" = \"Sri Lanka\";\n"
-"\"Started at %s\" = \"Started at %s\";\n"
-"\"State/Province\" = \"State/Province\";\n"
+"\"Started %s\" = \"Started %s\";\n"
 "\"Status\" = \"Status\";\n"
 "\"Sub-commands:\" = \"Sub-commands:\";\n"
 "\"Sudan\" = \"Sudan\";\n"
@@ -489,9 +493,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to create certificate.\" = \"Unable to create certificate.\";\n"
 "\"Unable to create test print job.\" = \"Unable to create test print job.\";\n"
 "\"Unable to identify printer.\" = \"Unable to identify printer.\";\n"
-"\"Unable to join Wi-Fi network.\" = \"Unable to join Wi-Fi network.\";\n"
 "\"Unable to lookup address.\" = \"Unable to lookup address.\";\n"
-"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Unable to use that driver.\";\n"
 "\"United Arab Emirates\" = \"United Arab Emirates\";\n"
 "\"United Kingdom\" = \"United Kingdom\";\n"
@@ -502,6 +504,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unknown action.\" = \"Unknown action.\";\n"
 "\"Unknown form action.\" = \"Unknown form action.\";\n"
 "\"Unknown form request.\" = \"Unknown form request.\";\n"
+"\"Updating Network Configuration\" = \"Updating Network Configuration\";\n"
 "\"Uruguay\" = \"Uruguay\";\n"
 "\"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\" = \"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\";\n"
 "\"Use My Position\" = \"Use My Position\";\n"
@@ -524,6 +527,7 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Yemen\" = \"Yemen\";\n"
 "\"Zambia\" = \"Zambia\";\n"
 "\"Zimbabwe\" = \"Zimbabwe\";\n"
+"\"at %02d:%02d:%02d\" = \"at %02d:%02d:%02d\";\n"
 /* IPP attribute, keyword, and enum (integer) localizations */
 "\"copies\" = \"Number of Copies\";\n"
 "\"default printer\" = \"default printer\";\n"
@@ -1057,4 +1061,5 @@ static const char *en_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"sides.two-sided-long-edge\" = \"On (portrait)\";\n"
 "\"sides.two-sided-short-edge\" = \"On (landscape)\";\n"
 "\"unable to join\" = \"unable to join\";\n"
+"\"yesterday at %02d:%02d:%02d\" = \"yesterday at %02d:%02d:%02d\";\n"
 "\"Åland Islands\" = \"Åland Islands\";\n";
