@@ -1,5 +1,5 @@
-static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\" = \"  -a               Cancelar todos los trabajos (cancel.)\";\n"
-"\"  -c COPIES\" = \"-c COPIES\";\n"
+static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\" = \"  -a               Cancelar todos los trabajos (cancelar).\";\n"
+"\"  -c COPIES\" = \"  -c COPIAS\";\n"
 "\"  -d PRINTER       Specify printer.\" = \"  -d PRINTER       Especifique la impresora.\";\n"
 "\"  -j JOB-ID        Specify job ID (cancel).\" = \"  -j JOB-ID        Especifique el ID de trabajo (cancel.)\";\n"
 "\"  -m DRIVER-NAME   Specify driver (add/modify).\" = \"  -m DRIVER-NAME   Especificar controlador (add/modify.)\";\n"
@@ -31,6 +31,8 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"  shutdown         Shutdown a running server.\" = \"  shutdown          Apaga un servidor en funcionamiento.\";\n"
 "\"  status           Show server/printer/job status.\" = \"  status           Mostrar el estado del servidor/printer/job.\";\n"
 "\"  submit           Submit a file for printing.\" = \"  submit           Enviar un archivo para imprimir.\";\n"
+"\"%04d-%02d-%02d at %02d:%02d:%02d\" = \"%04d-%02d-%02d at %02d:%02d:%02d\";\n"
+"\"%d days ago at %02d:%02d:%02d\" = \"hace %d días a las %02d:%02d:%02d\";\n"
 "\"%d inches/sec\" = \"%d pulgadas/sec\";\n"
 "\"%d job\" = \"%d trabajo\";\n"
 /* Media size in millimeters */
@@ -72,13 +74,13 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Sub-command 'autoadd' is not supported.\" = \"%s: Autoadd subcommand no es compatible.\";\n"
 "\"%s: Too many files.\" = \"%s: Demasiados archivos.\";\n"
 "\"%s: Unable to access '%s': %s\" = \"%s: Incapaz de acceder a '%s': %s\";\n"
-"\"%s: Unable to add '%s': %s\" = \"%s: Incapaz de añadir '%s': %s\";\n"
 "\"%s: Unable to add printer: %s\" = \"%s: Incapaz de añadir la impresora: %s\";\n"
 "\"%s: Unable to cancel job: %s\" = \"%s: Incapaz de cancelar el trabajo: %s\";\n"
 "\"%s: Unable to connect to printer at '%s:%d': %s\" = \"%s: Incapaz de conectarse a la impresora en '%s:%d': %s\";\n"
 "\"%s: Unable to connect to server: %s\" = \"%s: Incapaz de conectarse al servidor: %s\";\n"
 "\"%s: Unable to create temporary file: %s\" = \"%s: Incapaz de crear un archivo temporal: %s\";\n"
 "\"%s: Unable to delete printer: %s\" = \"%s: Incapaz de eliminar la impresora: %s\";\n"
+"\"%s: Unable to get available devices: %s\" = \"%s: No se pueden obtener los dispositivos disponibles: %s\";\n"
 "\"%s: Unable to get information for '%s': %s\" = \"%s: Incapaz de obtener información para '%s': %s\";\n"
 "\"%s: Unable to get information for printer: %s\" = \"%s: Incapaz de obtener información para la impresora: %s\";\n"
 "\"%s: Unable to get printer options: %s\" = \"%s: Incapaz de obtener opciones de impresora: %s\";\n"
@@ -97,7 +99,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Warning - user credentials are not supported in URIs.\" = \"%s: Advertencia - las credenciales del usuario no son compatibles con URIs.\";\n"
 "\", Borderless\" = \", sin fronteras\";\n"
 "\"A printer with that name already exists.\" = \"Una impresora con ese nombre ya existe.\";\n"
-"\"Aborted at %s\" = \"Abortado en %s\";\n"
+"\"Aborted %s\" = \"Abortado %s\";\n"
 "\"Add Printer\" = \"Añadir impresora\";\n"
 "\"Admin Group\" = \"Admin Group\";\n"
 "\"Afghanistan\" = \"Afganistán\";\n"
@@ -149,7 +151,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Canada\" = \"Canadá\";\n"
 "\"Cancel All Jobs\" = \"Cancelar Todos los trabajos\";\n"
 "\"Cancel Job\" = \"Cancelar trabajo\";\n"
-"\"Canceled at %s\" = \"Cancelado en %s\";\n"
+"\"Canceled %s\" = \"Cancelado %s\";\n"
 "\"Canceling\" = \"Cancelación\";\n"
 "\"Cayman Islands\" = \"Islas Caimán\";\n"
 "\"Central African Republic\" = \"República Centroafricana\";\n"
@@ -159,17 +161,18 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chad\" = \"Chad\";\n"
 "\"Change Access Password\" = \"Cambiar contraseña\";\n"
 "\"Change Log Level\" = \"Nivel de registro de cambios\";\n"
-"\"Change Network Settings\" = \"Change Network Settings\";\n"
+"\"Change Network Settings\" = \"Cambiar la configuración de la red\";\n"
 "\"Change Wi-Fi Network\" = \"Cambiar red Wi-Fi\";\n"
 "\"Changes Saved.\" = \"Cambios guardados.\";\n"
 "\"Changes saved.\" = \"Cambios salvados.\";\n"
 "\"Chile\" = \"Chile\";\n"
 "\"China\" = \"China\";\n"
 "\"Christmas Island\" = \"Isla de Navidad\";\n"
+"\"City/Locality\" = \"Ciudad/Localidad\";\n"
 "\"Cocos (Keeling) Islands\" = \"Islas Cocos (Keeling)\";\n"
 "\"Colombia\" = \"Colombia\";\n"
 "\"Comoros\" = \"Comoras\";\n"
-"\"Completed at %s\" = \"Completado en %s\";\n"
+"\"Completed %s\" = \"Completado %s\";\n"
 "\"Configuration\" = \"Configuración\";\n"
 "\"Configure Media\" = \"Configure Media\";\n"
 "\"Confirm Cancel All\" = \"Confirmación Todos\";\n"
@@ -192,7 +195,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Czechia\" = \"Czechia\";\n"
 "\"Côte d'Ivoire\" = \"Cte d'Ivoire\";\n"
 "\"DHCP\" = \"DHCP\";\n"
-"\"DHCP w/Manual Address\" = \"DHCP w/Manual Address\";\n"
+"\"DHCP w/Manual Address\" = \"DHCP con dirección manual\";\n"
 "\"DNS-SD Service Name\" = \"DNS-SD Nombre del servicio\";\n"
 "\"Debugging\" = \"Debugging\";\n"
 "\"Default printer set.\" = \"Juego de impresora predeterminado.\";\n"
@@ -200,7 +203,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Delete Printer\" = \"Eliminar la impresora\";\n"
 "\"Denmark\" = \"Dinamarca\";\n"
 "\"Djibouti\" = \"Djibouti\";\n"
-"\"Domain Name\" = \"Domain Name\";\n"
+"\"Domain Name\" = \"Nombre del dominio\";\n"
 "\"Dominica\" = \"Dominica\";\n"
 "\"Dominican Republic\" = \"República Dominicana\";\n"
 "\"Download Certificate Request File\" = \"Descargar certificado Solicitar archivo\";\n"
@@ -241,39 +244,40 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haití\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Island y McDonald Islands\";\n"
-"\"Hold Job\" = \"Hold Job\";\n"
-"\"Hold New Jobs\" = \"Hold New Jobs\";\n"
-"\"Holding new jobs.\" = \"Holding new jobs.\";\n"
+"\"Hidden SSID\" = \"SSID oculto\";\n"
+"\"Hold Job\" = \"Mantener el empleo\";\n"
+"\"Hold New Jobs\" = \"Mantener los nuevos empleos\";\n"
+"\"Holding new jobs.\" = \"Mantener nuevos puestos de trabajo.\";\n"
 "\"Holy See\" = \"Santa Sede\";\n"
 "\"Honduras\" = \"Honduras\";\n"
 "\"Hong Kong\" = \"Hong Kong\";\n"
 "\"Hostname\" = \"Nombre de host\";\n"
 "\"Human-Readable Location\" = \"Ubicación legible por humanos\";\n"
 "\"Hungary\" = \"Hungría\";\n"
-"\"IPv4 Address\" = \"IPv4 Address\";\n"
-"\"IPv4 Configuration\" = \"IPv4 Configuration\";\n"
-"\"IPv4 Gateway\" = \"IPv4 Gateway\";\n"
-"\"IPv4 Netmask\" = \"IPv4 Netmask\";\n"
-"\"IPv6 Address\" = \"IPv6 Address\";\n"
-"\"IPv6 Configuration\" = \"IPv6 Configuration\";\n"
-"\"IPv6 Link-Local\" = \"IPv6 Link-Local\";\n"
-"\"IPv6 Prefix Length\" = \"IPv6 Prefix Length\";\n"
+"\"IPv4 Address\" = \"Dirección IPv4\";\n"
+"\"IPv4 Configuration\" = \"Configuración IPv4\";\n"
+"\"IPv4 Gateway\" = \"Puerta de enlace IPv4\";\n"
+"\"IPv4 Netmask\" = \"Máscara de red IPv4\";\n"
+"\"IPv6 Address\" = \"Direcciones IPv6\";\n"
+"\"IPv6 Configuration\" = \"Configuración IPv6\";\n"
+"\"IPv6 Link-Local\" = \"Enlace local IPv6\";\n"
+"\"IPv6 Prefix Length\" = \"Longitud del prefijo IPv6\";\n"
 "\"Iceland\" = \"Islandia\";\n"
 "\"Identify Printer\" = \"Identifique la impresora\";\n"
 "\"India\" = \"India\";\n"
 "\"Indonesia\" = \"Indonesia\";\n"
 "\"Informational\" = \"Información\";\n"
 "\"Install TLS Certificate\" = \"Instala TLS Certificado\";\n"
-"\"Invalid IPv4 address.\" = \"Invalid IPv4 address.\";\n"
-"\"Invalid IPv4 netmask.\" = \"Invalid IPv4 netmask.\";\n"
-"\"Invalid IPv6 address.\" = \"Invalid IPv6 address.\";\n"
-"\"Invalid IPv6 prefix length.\" = \"Invalid IPv6 prefix length.\";\n"
+"\"Invalid IPv4 address.\" = \"Dirección IPv4 incorrecta.\";\n"
+"\"Invalid IPv4 netmask.\" = \"Máscara de red IPv4 incorrecta.\";\n"
+"\"Invalid IPv6 address.\" = \"Dirección IPv6 incorrecta.\";\n"
+"\"Invalid IPv6 prefix length.\" = \"Longitud del prefijo IPv6 incorrecto.\";\n"
 "\"Invalid Job ID.\" = \"ID de trabajo inválido.\";\n"
 "\"Invalid certificate or private key.\" = \"Certificado inválido o clave privada.\";\n"
 "\"Invalid form data.\" = \"Datos de formulario inválidos.\";\n"
 "\"Invalid form submission.\" = \"Sumisión de formulario inválido.\";\n"
 "\"Invalid job ID.\" = \"ID de trabajo inválido.\";\n"
-"\"Invalid network configuration.\" = \"Invalid network configuration.\";\n"
+"\"Invalid network configuration.\" = \"Configuración de red incorrecta.\";\n"
 "\"Iran (Islamic Republic of)\" = \"Irán (República Islámica del)\";\n"
 "\"Iraq\" = \"Iraq\";\n"
 "\"Ireland\" = \"Irlanda\";\n"
@@ -284,10 +288,11 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Japan\" = \"Japón\";\n"
 "\"Jersey\" = \"Jersey\";\n"
 "\"Job #\" = \"Job\";\n"
-"\"Job canceled.\" = \"Job canceled.\";\n"
-"\"Job held.\" = \"Job held.\";\n"
-"\"Job released.\" = \"Job released.\";\n"
+"\"Job canceled.\" = \"Trabajo cancelado.\";\n"
+"\"Job held.\" = \"Puesto ocupado.\";\n"
+"\"Job released.\" = \"Trabajo liberado.\";\n"
 "\"Jobs\" = \"Empleo\";\n"
+"\"Joining Wi-Fi Network\" = \"Unirse a la red Wi-Fi\";\n"
 "\"Jordan\" = \"Jordania\";\n"
 "\"Kazakhstan\" = \"Kazajstán\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -318,7 +323,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Maldives\" = \"Maldivas\";\n"
 "\"Mali\" = \"Malí\";\n"
 "\"Malta\" = \"Malta\";\n"
-"\"Manual Configuration\" = \"Manual Configuration\";\n"
+"\"Manual Configuration\" = \"Configuración manual\";\n"
 "\"Marshall Islands\" = \"Islas Marshall\";\n"
 "\"Martinique\" = \"Martinica\";\n"
 "\"Mauritania\" = \"Mauritania\";\n"
@@ -328,7 +333,8 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Mexico\" = \"México\";\n"
 "\"Micronesia (Federated States of)\" = \"Micronesia (Estados Federados de)\";\n"
 "\"Missing action.\" = \"Falta de acción.\";\n"
-"\"Missing job ID.\" = \"Missing job ID.\";\n"
+"\"Missing job ID.\" = \"Falta el identificador del trabajo.\";\n"
+"\"Missing network name.\" = \"Falta el nombre de la red.\";\n"
 "\"Missing private key.\" = \"Falta la llave privada.\";\n"
 "\"Moldova, Republic of\" = \"República de Moldova\";\n"
 "\"Monaco\" = \"Mónaco\";\n"
@@ -359,7 +365,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Northern Mariana Islands\" = \"Islas Marianas del Norte\";\n"
 "\"Norway\" = \"Noruega\";\n"
 "\"Not set\" = \"No está listo.\";\n"
-"\"Off\" = \"Off\";\n"
+"\"Off\" = \"Apagado\";\n"
 "\"Offset\" = \"Offset\";\n"
 "\"Oman\" = \"Omán\";\n"
 "\"Options:\" = \"Opciones:\";\n"
@@ -388,7 +394,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Poland\" = \"Polonia\";\n"
 "\"Portrait\" = \"Retrato\";\n"
 "\"Portugal\" = \"Portugal\";\n"
-"\"Primary DNS\" = \"Primary DNS\";\n"
+"\"Primary DNS\" = \"DNS principal\";\n"
 "\"Print Group\" = \"Print Group\";\n"
 "\"Print Test Page\" = \"Página de prueba de impresión\";\n"
 "\"Print job options:\" = \"Opciones de trabajo para imprimir:\";\n"
@@ -403,13 +409,12 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Printing Defaults\" = \"Impresos\";\n"
 "\"Puerto Rico\" = \"Puerto Rico\";\n"
 "\"Qatar\" = \"Qatar\";\n"
-"\"Queued at %s\" = \"Queued at %s\";\n"
-"\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
-"\"Release Job\" = \"Release Job\";\n"
-"\"Released held new jobs.\" = \"Released held new jobs.\";\n"
+"\"Queued %s\" = \"En espera %s\";\n"
+"\"Release Held New Jobs\" = \"Liberación de nuevos puestos de trabajo\";\n"
+"\"Release Job\" = \"Liberar el trabajo\";\n"
+"\"Released held new jobs.\" = \"Se han reanudado los nuevos trabajos en pausa.\";\n"
 "\"Reprint Job\" = \"Reprint Job\";\n"
-"\"Reprinted job.\" = \"Reprinted job.\";\n"
-"\"Rescan\" = \"Rescan\";\n"
+"\"Reprinted job.\" = \"Trabajo reeditado.\";\n"
 "\"Resume Printing\" = \"Resume Printing\";\n"
 "\"Reverse Landscape\" = \"Paisaje inverso\";\n"
 "\"Reverse Portrait\" = \"Retrato inverso\";\n"
@@ -431,7 +436,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Sao Tome and Principe\" = \"Santo Tomé y Príncipe\";\n"
 "\"Saudi Arabia\" = \"Arabia Saudita\";\n"
 "\"Save Changes\" = \"Guardar cambios\";\n"
-"\"Secondary DNS\" = \"Secondary DNS\";\n"
+"\"Secondary DNS\" = \"DNS secundarias\";\n"
 "\"Security\" = \"Seguridad\";\n"
 "\"Select Driver\" = \"Select Driver\";\n"
 "\"Senegal\" = \"Senegal\";\n"
@@ -452,8 +457,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"South Sudan\" = \"Sudán del Sur\";\n"
 "\"Spain\" = \"España\";\n"
 "\"Sri Lanka\" = \"Sri Lanka\";\n"
-"\"Started at %s\" = \"Comenzó a %s\";\n"
-"\"State/Province\" = \"State/Province\";\n"
+"\"Started %s\" = \"Empezó %s\";\n"
 "\"Status\" = \"Situación\";\n"
 "\"Sub-commands:\" = \"Subcomandantes:\";\n"
 "\"Sudan\" = \"Sudán\";\n"
@@ -489,9 +493,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to create certificate.\" = \"Incapaz de crear certificado.\";\n"
 "\"Unable to create test print job.\" = \"Incapaz de crear trabajo de impresión de prueba.\";\n"
 "\"Unable to identify printer.\" = \"Incapaz de identificar la impresora.\";\n"
-"\"Unable to join Wi-Fi network.\" = \"Incapaz de unirse a la red Wi-Fi.\";\n"
 "\"Unable to lookup address.\" = \"Incapaz de buscar la dirección.\";\n"
-"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"Incapaz de usar ese conductor.\";\n"
 "\"United Arab Emirates\" = \"Emiratos Árabes Unidos\";\n"
 "\"United Kingdom\" = \"Reino Unido\";\n"
@@ -502,6 +504,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unknown action.\" = \"Acción desconocida.\";\n"
 "\"Unknown form action.\" = \"Acción de forma desconocida.\";\n"
 "\"Unknown form request.\" = \"Solicitud de formulario desconocido.\";\n"
+"\"Updating Network Configuration\" = \"Actualizar la configuración de red\";\n"
 "\"Uruguay\" = \"Uruguay\";\n"
 "\"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\" = \"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\";\n"
 "\"Use My Position\" = \"Usar mi posición\";\n"
@@ -524,6 +527,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Yemen\" = \"Yemen\";\n"
 "\"Zambia\" = \"Zambia\";\n"
 "\"Zimbabwe\" = \"Zimbabwe\";\n"
+"\"at %02d:%02d:%02d\" = \"en %02d:%02d:%02d\";\n"
 /* IPP attribute, keyword, and enum (integer) localizations */
 "\"copies\" = \"Número de copias\";\n"
 "\"default printer\" = \"impresora predeterminada\";\n"
@@ -597,7 +601,7 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"finishings.97\" = \"Pliegue paralelo\";\n"
 "\"finishings.98\" = \"Cargador\";\n"
 "\"finishings.99\" = \"Puerta derecha\";\n"
-"\"holding new jobs\" = \"holding new jobs\";\n"
+"\"holding new jobs\" = \"ocupar nuevos puestos\";\n"
 "\"job-state-reasons.aborted-by-system\" = \"Para.\";\n"
 "\"job-state-reasons.compression-error\" = \"Error de descompresión.\";\n"
 "\"job-state-reasons.document-format-error\" = \"Error de formato de documento.\";\n"
@@ -1057,4 +1061,5 @@ static const char *es_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"sides.two-sided-long-edge\" = \"En (portrait)\";\n"
 "\"sides.two-sided-short-edge\" = \"En (paisaje)\";\n"
 "\"unable to join\" = \"incapaz de unirse\";\n"
+"\"yesterday at %02d:%02d:%02d\" = \"ayer a las %02d:%02d:%02d\";\n"
 "\"Åland Islands\" = \"Islas\";\n";
