@@ -31,6 +31,8 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"  shutdown         Shutdown a running server.\" = \"  shutdown 実行中のサーバーをシャットダウンします。\";\n"
 "\"  status           Show server/printer/job status.\" = \"  status ステータス サーバ/プリンター/ジョブの状態を表示します。\";\n"
 "\"  submit           Submit a file for printing.\" = \"  submit 印刷用のファイルを提出してください。\";\n"
+"\"%04d-%02d-%02d at %02d:%02d:%02d\" = \"%04d-%02d-%02d at %02d:%02d:%02d\";\n"
+"\"%d days ago at %02d:%02d:%02d\" = \"%d days ago at %02d:%02d:%02d\";\n"
 "\"%d inches/sec\" = \"%dインチ/秒\";\n"
 "\"%d job\" = \"%d ジョブ\";\n"
 /* Media size in millimeters */
@@ -72,13 +74,13 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Sub-command 'autoadd' is not supported.\" = \"%s: サブコマンドの自動追加はサポートされていません。\";\n"
 "\"%s: Too many files.\" = \"%s: あまりにも多くのファイル。\";\n"
 "\"%s: Unable to access '%s': %s\" = \"%s: '%s': %s にアクセスできない\";\n"
-"\"%s: Unable to add '%s': %s\" = \"%s: '%s': %s を追加できません。\";\n"
 "\"%s: Unable to add printer: %s\" = \"%s: プリンターを追加できません: %s\";\n"
 "\"%s: Unable to cancel job: %s\" = \"%s: ジョブをキャンセルできません: %s\";\n"
 "\"%s: Unable to connect to printer at '%s:%d': %s\" = \"%s: プリンターに接続できない '%s:%d': %s\";\n"
 "\"%s: Unable to connect to server: %s\" = \"%s: サーバーに接続できません: %s\";\n"
 "\"%s: Unable to create temporary file: %s\" = \"%s: 一時ファイルを作成することができません: %s\";\n"
 "\"%s: Unable to delete printer: %s\" = \"%s: プリンターを削除できません: %s\";\n"
+"\"%s: Unable to get available devices: %s\" = \"%s: Unable to get available devices: %s\";\n"
 "\"%s: Unable to get information for '%s': %s\" = \"%s: '%s': %s の情報を取得することができません\";\n"
 "\"%s: Unable to get information for printer: %s\" = \"%s: プリンター情報を取得することができません: %s\";\n"
 "\"%s: Unable to get printer options: %s\" = \"%s: プリンタのオプションを取得することができません: %s\";\n"
@@ -97,7 +99,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Warning - user credentials are not supported in URIs.\" = \"%s: 警告 - ユーザ認証情報は URI でサポートされていません。\";\n"
 "\", Borderless\" = \", ボーダーレス\";\n"
 "\"A printer with that name already exists.\" = \"既に存在している名前のプリンター。\";\n"
-"\"Aborted at %s\" = \"%sで中止\";\n"
+"\"Aborted %s\" = \"Aborted %s\";\n"
 "\"Add Printer\" = \"プリンターを追加\";\n"
 "\"Admin Group\" = \"管理者グループ\";\n"
 "\"Afghanistan\" = \"アフガニスタン\";\n"
@@ -149,7 +151,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Canada\" = \"カナダ\";\n"
 "\"Cancel All Jobs\" = \"すべてのジョブをキャンセルする\";\n"
 "\"Cancel Job\" = \"キャンセルジョブ\";\n"
-"\"Canceled at %s\" = \"キャンセル %s\";\n"
+"\"Canceled %s\" = \"Canceled %s\";\n"
 "\"Canceling\" = \"キャンセル\";\n"
 "\"Cayman Islands\" = \"ケイマン諸島\";\n"
 "\"Central African Republic\" = \"中央アフリカ共和国\";\n"
@@ -166,10 +168,11 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chile\" = \"チリ\";\n"
 "\"China\" = \"中国の\";\n"
 "\"Christmas Island\" = \"クリスマス島\";\n"
+"\"City/Locality\" = \"City/Locality\";\n"
 "\"Cocos (Keeling) Islands\" = \"ココス諸島(ケイリング)\";\n"
 "\"Colombia\" = \"コロンビア\";\n"
 "\"Comoros\" = \"コモロ\";\n"
-"\"Completed at %s\" = \"%s で完了\";\n"
+"\"Completed %s\" = \"Completed %s\";\n"
 "\"Configuration\" = \"コンテンツ\";\n"
 "\"Configure Media\" = \"メディアの設定\";\n"
 "\"Confirm Cancel All\" = \"キャンセル確認 すべて\";\n"
@@ -241,6 +244,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"ガイアナ\";\n"
 "\"Haiti\" = \"ハイチ\";\n"
 "\"Heard Island and McDonald Islands\" = \"ヘアード島とマクドナルド諸島\";\n"
+"\"Hidden SSID\" = \"Hidden SSID\";\n"
 "\"Hold Job\" = \"Hold Job\";\n"
 "\"Hold New Jobs\" = \"Hold New Jobs\";\n"
 "\"Holding new jobs.\" = \"Holding new jobs.\";\n"
@@ -288,6 +292,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Job held.\" = \"Job held.\";\n"
 "\"Job released.\" = \"Job released.\";\n"
 "\"Jobs\" = \"求人情報\";\n"
+"\"Joining Wi-Fi Network\" = \"Joining Wi-Fi Network\";\n"
 "\"Jordan\" = \"ヨルダン\";\n"
 "\"Kazakhstan\" = \"カザフスタン\";\n"
 "\"Kenya\" = \"ケニア\";\n"
@@ -329,6 +334,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Micronesia (Federated States of)\" = \"マイクロネシア(連邦)\";\n"
 "\"Missing action.\" = \"行動を欠く。\";\n"
 "\"Missing job ID.\" = \"Missing job ID.\";\n"
+"\"Missing network name.\" = \"Missing network name.\";\n"
 "\"Missing private key.\" = \"プライベートキーを欠く。\";\n"
 "\"Moldova, Republic of\" = \"モルドバ共和国\";\n"
 "\"Monaco\" = \"メニュー\";\n"
@@ -403,13 +409,12 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Printing Defaults\" = \"印刷のデフォルト\";\n"
 "\"Puerto Rico\" = \"プエルトリコ\";\n"
 "\"Qatar\" = \"カタール\";\n"
-"\"Queued at %s\" = \"%sでキューイング\";\n"
+"\"Queued %s\" = \"Queued %s\";\n"
 "\"Release Held New Jobs\" = \"Release Held New Jobs\";\n"
 "\"Release Job\" = \"Release Job\";\n"
 "\"Released held new jobs.\" = \"Released held new jobs.\";\n"
 "\"Reprint Job\" = \"再印刷ジョブ\";\n"
 "\"Reprinted job.\" = \"Reprinted job.\";\n"
-"\"Rescan\" = \"リスカ\";\n"
 "\"Resume Printing\" = \"再開の印刷\";\n"
 "\"Reverse Landscape\" = \"逆の風景\";\n"
 "\"Reverse Portrait\" = \"逆の肖像\";\n"
@@ -452,8 +457,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"South Sudan\" = \"南スーダン\";\n"
 "\"Spain\" = \"スペイン\";\n"
 "\"Sri Lanka\" = \"スリランカ\";\n"
-"\"Started at %s\" = \"開始 %s\";\n"
-"\"State/Province\" = \"都道府県/都道府県\";\n"
+"\"Started %s\" = \"Started %s\";\n"
 "\"Status\" = \"ステータス\";\n"
 "\"Sub-commands:\" = \"サブコマンド:\";\n"
 "\"Sudan\" = \"スーダン\";\n"
@@ -489,9 +493,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to create certificate.\" = \"証明書を作成することができません。\";\n"
 "\"Unable to create test print job.\" = \"テストプリントジョブを作成することができません。\";\n"
 "\"Unable to identify printer.\" = \"プリンターを識別できません。\";\n"
-"\"Unable to join Wi-Fi network.\" = \"Wi-Fiネットワークへの参加はできません。\";\n"
 "\"Unable to lookup address.\" = \"アドレスを調べることができません。\";\n"
-"\"Unable to save network configuration.\" = \"Unable to save network configuration.\";\n"
 "\"Unable to use that driver.\" = \"そのドライバを使用できません。\";\n"
 "\"United Arab Emirates\" = \"アラブ首長国連邦\";\n"
 "\"United Kingdom\" = \"イギリス\";\n"
@@ -502,6 +504,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unknown action.\" = \"未知のアクション。\";\n"
 "\"Unknown form action.\" = \"未知のフォームアクション。\";\n"
 "\"Unknown form request.\" = \"不明なフォームリクエスト\";\n"
+"\"Updating Network Configuration\" = \"Updating Network Configuration\";\n"
 "\"Uruguay\" = \"ウルグアイ\";\n"
 "\"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\" = \"使用法: %s サブコマンド [操作] [ユーザー名]\\n\";\n"
 "\"Use My Position\" = \"マイポジションを使用する\";\n"
@@ -524,6 +527,7 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Yemen\" = \"イエメン\";\n"
 "\"Zambia\" = \"ザンビア\";\n"
 "\"Zimbabwe\" = \"ジンバブエ\";\n"
+"\"at %02d:%02d:%02d\" = \"at %02d:%02d:%02d\";\n"
 /* IPP attribute, keyword, and enum (integer) localizations */
 "\"copies\" = \"コピーの数\";\n"
 "\"default printer\" = \"デフォルトプリンタ\";\n"
@@ -1057,4 +1061,5 @@ static const char *ja_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"sides.two-sided-long-edge\" = \"(ポートレート)\";\n"
 "\"sides.two-sided-short-edge\" = \"オン(ランドスケープ)\";\n"
 "\"unable to join\" = \"参加できません\";\n"
+"\"yesterday at %02d:%02d:%02d\" = \"yesterday at %02d:%02d:%02d\";\n"
 "\"Åland Islands\" = \"土地の島\";\n";
