@@ -31,6 +31,8 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"  shutdown         Shutdown a running server.\" = \"  shutdown         Çalışan bir sunucuyu kapat.\";\n"
 "\"  status           Show server/printer/job status.\" = \"  status           Sunucu/yazıcı/iş durumunu göster.\";\n"
 "\"  submit           Submit a file for printing.\" = \"  submit           Yazdırmak için bir dosya gönder.\";\n"
+"\"%04d-%02d-%02d at %02d:%02d:%02d\" = \"%04d-%02d-%02d, %02d:%02d:%02d\";\n"
+"\"%d days ago at %02d:%02d:%02d\" = \"%d gün önce, %02d:%02d:%02d'de\";\n"
 "\"%d inches/sec\" = \"%d inç/s\";\n"
 "\"%d job\" = \"%d iş\";\n"
 /* Media size in millimeters */
@@ -72,13 +74,13 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Sub-command 'autoadd' is not supported.\" = \"%s: 'autoadd' alt komutu desteklenmiyor.\";\n"
 "\"%s: Too many files.\" = \"%s: Çok fazla dosya.\";\n"
 "\"%s: Unable to access '%s': %s\" = \"%s: '%s' erişilebilir değil: %s\";\n"
-"\"%s: Unable to add '%s': %s\" = \"%s: '%s' eklenemiyor: %s\";\n"
 "\"%s: Unable to add printer: %s\" = \"%s: Yazıcı eklenemiyor: %s\";\n"
 "\"%s: Unable to cancel job: %s\" = \"%s: İş iptal edilemiyor: %s\";\n"
 "\"%s: Unable to connect to printer at '%s:%d': %s\" = \"%s: '%s:%d' konumunda yazıcıya bağlanılamıyor: %s\";\n"
 "\"%s: Unable to connect to server: %s\" = \"%s: Sunucuya bağlanılamıyor: %s\";\n"
 "\"%s: Unable to create temporary file: %s\" = \"%s: Geçici dosya oluşturulamıyor: %s\";\n"
 "\"%s: Unable to delete printer: %s\" = \"%s: Yazıcı silinemiyor: %s\";\n"
+"\"%s: Unable to get available devices: %s\" = \"%s: Kullanılabilir aygıtlar alınamıyor: %s\";\n"
 "\"%s: Unable to get information for '%s': %s\" = \"%s: '%s' için bilgi alınamıyor: %s\";\n"
 "\"%s: Unable to get information for printer: %s\" = \"%s: Yazıcı için bilgi alınamıyor: %s\";\n"
 "\"%s: Unable to get printer options: %s\" = \"%s: Yazıcı seçenekleri alınamıyor: %s\";\n"
@@ -97,7 +99,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"%s: Warning - user credentials are not supported in URIs.\" = \"%s: Uyarı - URI'lerde kullanıcı kimlik bilgileri desteklenmiyor.\";\n"
 "\", Borderless\" = \", Kenarlıksız\";\n"
 "\"A printer with that name already exists.\" = \"Bu ada sahip bir yazıcı zaten var.\";\n"
-"\"Aborted at %s\" = \"%s'de iptal edildi\";\n"
+"\"Aborted %s\" = \"%s sonlandırıldı\";\n"
 "\"Add Printer\" = \"Yazıcı Ekle\";\n"
 "\"Admin Group\" = \"Yönetici Grubu\";\n"
 "\"Afghanistan\" = \"Afganistan\";\n"
@@ -149,7 +151,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Canada\" = \"Kanada\";\n"
 "\"Cancel All Jobs\" = \"Tüm İşleri İptal Et\";\n"
 "\"Cancel Job\" = \"İşi İptal Et\";\n"
-"\"Canceled at %s\" = \"%s'de iptal edildi\";\n"
+"\"Canceled %s\" = \"%s iptal edildi\";\n"
 "\"Canceling\" = \"İptal ediliyor\";\n"
 "\"Cayman Islands\" = \"Cayman Adaları\";\n"
 "\"Central African Republic\" = \"Orta Afrika Cumhuriyeti\";\n"
@@ -166,10 +168,11 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Chile\" = \"Şili\";\n"
 "\"China\" = \"Çin\";\n"
 "\"Christmas Island\" = \"Christmas Adası\";\n"
+"\"City/Locality\" = \"Şehir/Yerellik\";\n"
 "\"Cocos (Keeling) Islands\" = \"Cocos (Keeling) Adaları\";\n"
 "\"Colombia\" = \"Kolombiya\";\n"
 "\"Comoros\" = \"Komorlar\";\n"
-"\"Completed at %s\" = \"%s'de tamamlandı\";\n"
+"\"Completed %s\" = \"%s tamamlandı\";\n"
 "\"Configuration\" = \"Yapılandırma\";\n"
 "\"Configure Media\" = \"Ortamı Yapılandır\";\n"
 "\"Confirm Cancel All\" = \"Tümünü İptal Etmeyi Onayla\";\n"
@@ -241,6 +244,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Guyana\" = \"Guyana\";\n"
 "\"Haiti\" = \"Haiti\";\n"
 "\"Heard Island and McDonald Islands\" = \"Heard Adası ve McDonald Adaları\";\n"
+"\"Hidden SSID\" = \"Gizli SSID\";\n"
 "\"Hold Job\" = \"İşi Beklet\";\n"
 "\"Hold New Jobs\" = \"Yeni İşleri Beklet\";\n"
 "\"Holding new jobs.\" = \"Yeni işler bekletiliyor.\";\n"
@@ -288,6 +292,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Job held.\" = \"İş bekletildi.\";\n"
 "\"Job released.\" = \"İş serbest bırakıldı.\";\n"
 "\"Jobs\" = \"İşler\";\n"
+"\"Joining Wi-Fi Network\" = \"Wi-Fi Ağına Bağlanıyor\";\n"
 "\"Jordan\" = \"Ürdün\";\n"
 "\"Kazakhstan\" = \"Kazakistan\";\n"
 "\"Kenya\" = \"Kenya\";\n"
@@ -329,6 +334,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Micronesia (Federated States of)\" = \"Mikronezya (Federal Devletleri)\";\n"
 "\"Missing action.\" = \"Eksik eylem.\";\n"
 "\"Missing job ID.\" = \"İş kimliği eksik.\";\n"
+"\"Missing network name.\" = \"Eksik ağ adı.\";\n"
 "\"Missing private key.\" = \"Eksik özel anahtar.\";\n"
 "\"Moldova, Republic of\" = \"Moldova Cumhuriyeti\";\n"
 "\"Monaco\" = \"Monako\";\n"
@@ -403,13 +409,12 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Printing Defaults\" = \"Öntanımlı Yazdırma Ayarları\";\n"
 "\"Puerto Rico\" = \"Porto Riko\";\n"
 "\"Qatar\" = \"Katar\";\n"
-"\"Queued at %s\" = \"%s'de sıraya alındı\";\n"
+"\"Queued %s\" = \"%s sıraya alındı\";\n"
 "\"Release Held New Jobs\" = \"Bekletilen Yeni İşleri Serbest Bırak\";\n"
 "\"Release Job\" = \"İşi Serbest Bırak\";\n"
 "\"Released held new jobs.\" = \"Bekletilen yeni işler serbest bırakıldı.\";\n"
 "\"Reprint Job\" = \"İşi Yeniden Yazdır\";\n"
 "\"Reprinted job.\" = \"İş yeniden yazdırıldı.\";\n"
-"\"Rescan\" = \"Yeniden Tara\";\n"
 "\"Resume Printing\" = \"Yazdırmayı Devam Ettir\";\n"
 "\"Reverse Landscape\" = \"Ters Manzara\";\n"
 "\"Reverse Portrait\" = \"Ters Portre\";\n"
@@ -452,8 +457,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"South Sudan\" = \"Güney Sudan\";\n"
 "\"Spain\" = \"İspanya\";\n"
 "\"Sri Lanka\" = \"Sri Lanka\";\n"
-"\"Started at %s\" = \"%s'de başladı\";\n"
-"\"State/Province\" = \"Eyalet/İl\";\n"
+"\"Started %s\" = \"%s başlatıldı\";\n"
 "\"Status\" = \"Durum\";\n"
 "\"Sub-commands:\" = \"Alt komutlar:\";\n"
 "\"Sudan\" = \"Sudan\";\n"
@@ -489,9 +493,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unable to create certificate.\" = \"Sertifika oluşturulamıyor.\";\n"
 "\"Unable to create test print job.\" = \"Test yazdırma işi oluşturulamıyor.\";\n"
 "\"Unable to identify printer.\" = \"Yazıcı tanımlanamıyor.\";\n"
-"\"Unable to join Wi-Fi network.\" = \"Wi-Fi ağına katılınamıyor.\";\n"
 "\"Unable to lookup address.\" = \"Adres aranamıyor.\";\n"
-"\"Unable to save network configuration.\" = \"Ağ yapılandırması kaydedilemiyor.\";\n"
 "\"Unable to use that driver.\" = \"Bu sürücü kullanılamıyor.\";\n"
 "\"United Arab Emirates\" = \"Birleşik Arap Emirlikleri\";\n"
 "\"United Kingdom\" = \"Birleşik Krallık\";\n"
@@ -502,6 +504,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Unknown action.\" = \"Bilinmeyen eylem.\";\n"
 "\"Unknown form action.\" = \"Bilinmeyen form eylemi.\";\n"
 "\"Unknown form request.\" = \"Bilinmeyen form isteği.\";\n"
+"\"Updating Network Configuration\" = \"Ağ Yapılandırması Güncelleniyor\";\n"
 "\"Uruguay\" = \"Uruguay\";\n"
 "\"Usage: %s SUB-COMMAND [OPTIONS] [FILENAME]\\n\" = \"Kullanım: %s ALT-KOMUT [SEÇENEKLER] [DOSYA-ADI]\\n\";\n"
 "\"Use My Position\" = \"Konumumu Kullan\";\n"
@@ -524,6 +527,7 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"Yemen\" = \"Yemen\";\n"
 "\"Zambia\" = \"Zambiya\";\n"
 "\"Zimbabwe\" = \"Zimbabve\";\n"
+"\"at %02d:%02d:%02d\" = \"%02d:%02d:%02d\";\n"
 /* IPP attribute, keyword, and enum (integer) localizations */
 "\"copies\" = \"Kopya Sayısı\";\n"
 "\"default printer\" = \"öntanımlı yazıcı\";\n"
@@ -1057,4 +1061,5 @@ static const char *tr_strings = "\"  -a               Cancel all jobs (cancel).\
 "\"sides.two-sided-long-edge\" = \"Açık (portre)\";\n"
 "\"sides.two-sided-short-edge\" = \"Açık (manzara)\";\n"
 "\"unable to join\" = \"katılamıyor\";\n"
+"\"yesterday at %02d:%02d:%02d\" = \"dün, %02d:%02d:%02d\";\n"
 "\"Åland Islands\" = \"Åland Adaları\";\n";
