@@ -1,16 +1,12 @@
 //
 // System accessor functions for the Printer Application Framework
 //
-// Copyright © 2020-2022 by Michael R Sweet.
+// Copyright © 2020-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "system-private.h"
 #ifdef HAVE_LIBJPEG
 #  include <jpeglib.h>
@@ -136,6 +132,18 @@ papplSystemAddListeners(
   else
   {
     // Add named listeners on both IPv4 and IPv6...
+    if (name && strcasecmp(name, "*"))
+    {
+      // Listening on a specific hostname...
+      _papplRWLockWrite(system);
+
+      free(system->hostname);
+      system->hostname      = strdup(name);
+      system->is_listenhost = true;
+
+      _papplRWUnlock(system);
+    }
+
     if (system->port)
     {
       ret = add_listeners(system, name, system->port, AF_INET) ||
@@ -219,7 +227,7 @@ papplSystemAddMIMEFilter(
 //
 // This function schedules a function that will be called on the main run loop
 // thread at the specified time and optionally every "interval" seconds
-// thereafter.  The timimg accuracy is typically within a few milliseconds but
+// thereafter.  The timing accuracy is typically within a few milliseconds but
 // is not guaranteed.  Since the callback is run on the main run loop thread,
 // functions should create a new thread for any long-running operations.
 //
@@ -526,7 +534,17 @@ int					// O - "default-printer-id" value
 papplSystemGetDefaultPrinterID(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->default_printer_id : 0);
+  int ret = 0;				// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = system->default_printer_id;
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -774,7 +792,18 @@ pappl_loglevel_t			// O - Log level
 papplSystemGetLogLevel(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->loglevel : PAPPL_LOGLEVEL_UNSPEC);
+  pappl_loglevel_t ret = PAPPL_LOGLEVEL_UNSPEC;
+					// Return value
+
+
+  if (system)
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    ret = system->log_level;
+    pthread_mutex_unlock(&system->log_mutex);
+  }
+
+  return (ret);
 }
 
 
@@ -789,7 +818,17 @@ int					// O - Maximum number of clients
 papplSystemGetMaxClients(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->max_clients : 0);
+  int ret = 0;				// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = system->max_clients;
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -852,7 +891,17 @@ size_t					// O - Maximum log file size or `0` for none
 papplSystemGetMaxLogSize(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->logmaxsize : 0);
+  size_t ret = 0;			// Return value
+
+
+  if (system)
+  {
+    pthread_mutex_lock(&system->log_mutex);
+    ret = system->log_max_size;
+    pthread_mutex_unlock(&system->log_mutex);
+  }
+
+  return (ret);
 }
 
 
@@ -869,7 +918,17 @@ size_t					// O - Maximum number of subscriptions or `0`
 papplSystemGetMaxSubscriptions(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->max_subscriptions : 0);
+  size_t ret = 0;			// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = system->max_subscriptions;
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -920,7 +979,17 @@ int					// O - Next "printer-id" value
 papplSystemGetNextPrinterID(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->next_printer_id : 0);
+  int ret = 0;				// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = system->next_printer_id;
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -1095,26 +1164,22 @@ papplSystemGetSessionKey(
 
   if (system && buffer && bufsize > 0)
   {
+    pthread_mutex_lock(&system->session_mutex);
     if ((curtime - system->session_time) > 86400)
     {
       // Lock for updating the session key with random data...
-      pthread_rwlock_wrlock(&system->session_rwlock);
-
       snprintf(system->session_key, sizeof(system->session_key), "%08x%08x%08x%08x%08x%08x%08x%08x", papplGetRand(), papplGetRand(), papplGetRand(), papplGetRand(), papplGetRand(), papplGetRand(), papplGetRand(), papplGetRand());
       system->session_time = curtime;
     }
-    else
-    {
-      // Lock for reading...
-      pthread_rwlock_rdlock(&system->session_rwlock);
-    }
 
     papplCopyString(buffer, system->session_key, bufsize);
 
-    pthread_rwlock_unlock(&system->session_rwlock);
+    pthread_mutex_unlock(&system->session_mutex);
   }
   else if (buffer)
+  {
     *buffer = '\0';
+  }
 
   return (buffer);
 }
@@ -1145,7 +1210,17 @@ const char *				// O - "system-uuid" value
 papplSystemGetUUID(
     pappl_system_t *system)		// I - System
 {
-  return (system ? system->uuid : NULL);
+  const char *ret = NULL;		// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = system->uuid;
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -1214,7 +1289,7 @@ papplSystemHashPassword(
     {
       // Copy existing nonce from the salt string...
       papplCopyString(nonce, salt, sizeof(nonce));
-      if ((ptr = strchr(nonce, ':')) != NULL)
+      if ((ptr = strchr(nonce, '~')) != NULL)
         *ptr = '\0';
     }
     else
@@ -1260,6 +1335,21 @@ papplSystemIsRunning(
 }
 
 
+//
+// '_papplSystemIsShutdownNoLock()' - Return whether the system has been shutdown.
+//
+// This function returns whether the system is shutdown or scheduled to
+// shutdown.
+//
+
+bool					// O - `true` if the system is shutdown, `false` otherwise
+_papplSystemIsShutdownNoLock(
+    pappl_system_t *system)		// I - System
+{
+  return (!system->is_running || system->shutdown_time != 0);
+}
+
+
 //
 // 'papplSystemIsShutdown()' - Return whether the system has been shutdown.
 //
@@ -1271,7 +1361,17 @@ bool					// O - `true` if the system is shutdown, `false` otherwise
 papplSystemIsShutdown(
     pappl_system_t *system)		// I - System
 {
-  return (system ? (!system->is_running || system->shutdown_time != 0) : false);
+  bool ret = false;			// Return value
+
+
+  if (system)
+  {
+    _papplRWLockRead(system);
+    ret = _papplSystemIsShutdownNoLock(system);
+    _papplRWUnlock(system);
+  }
+
+  return (ret);
 }
 
 
@@ -1497,6 +1597,9 @@ papplSystemSetAdminGroup(
 // - `HTTP_STATUS_FORBIDDEN` if the authentication succeeded but the user is
 //   not part of the specified group.
 //
+// > Note: The authentication callback can only be set prior to calling
+// > @link papplSystemRun@.
+//
 
 void
 papplSystemSetAuthCallback(
@@ -1505,16 +1608,12 @@ papplSystemSetAuthCallback(
     pappl_auth_cb_t auth_cb,		// I - Callback function
     void            *auth_cbdata)	// I - Callback data
 {
-  if (system)
+  if (system && !system->is_running)
   {
-    _papplRWLockWrite(system);
-
     free(system->auth_scheme);
     system->auth_scheme = auth_scheme ? strdup(auth_scheme) : NULL;
     system->auth_cb     = auth_cb;
     system->auth_cbdata = auth_cbdata;
-
-    _papplRWUnlock(system);
   }
 }
 
@@ -1669,12 +1768,8 @@ papplSystemSetFooterHTML(
 {
   if (system && html && !system->is_running)
   {
-    _papplRWLockWrite(system);
-
     free(system->footer_html);
     system->footer_html = strdup(html);
-
-    _papplRWUnlock(system);
   }
 }
 
@@ -1695,7 +1790,7 @@ papplSystemSetGeoLocation(
 
 
   // Validate geo-location - must be NULL or a "geo:" URI...
-  if (value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
+  if (value && *value && sscanf(value, "geo:%f,%f", &lat, &lon) != 2)
     return;
 
   if (system)
@@ -1703,7 +1798,7 @@ papplSystemSetGeoLocation(
     _papplRWLockWrite(system);
 
     free(system->geo_location);
-    system->geo_location = value ? strdup(value) : NULL;
+    system->geo_location = value && *value ? strdup(value) : NULL;
 
     _papplSystemConfigChanged(system);
 
@@ -1748,6 +1843,9 @@ _papplSystemSetHostNameNoLock(
 	*ptr;				// Pointer in temporary hostname
 
 
+  if (system->is_listenhost)
+    return;
+
   if (value)
   {
 #if !defined(__APPLE__) && !_WIN32
@@ -1774,17 +1872,7 @@ _papplSystemSetHostNameNoLock(
   }
   else
   {
-#ifdef HAVE_AVAHI
-    _pappl_dns_sd_t	master = _papplDNSSDInit(system);
-					// DNS-SD master reference
-    const char *avahi_name = master ? avahi_client_get_host_name_fqdn(master) : NULL;
-					// mDNS hostname
-
-    if (avahi_name)
-      papplCopyString(temp, avahi_name, sizeof(temp));
-    else
-#endif /* HAVE_AVAHI */
-    httpGetHostname(NULL, temp, sizeof(temp));
+    _papplDNSSDCopyHostName(temp, sizeof(temp));
 
     if ((ptr = strstr(temp, ".lan")) != NULL && !ptr[4])
     {
@@ -1878,13 +1966,13 @@ papplSystemSetLogLevel(
 {
   if (system)
   {
-    _papplRWLockWrite(system);
+    pthread_mutex_lock(&system->log_mutex);
 
-    system->loglevel = loglevel;
+    system->log_level = loglevel;
 
     _papplSystemConfigChanged(system);
 
-    _papplRWUnlock(system);
+    pthread_mutex_unlock(&system->log_mutex);
   }
 }
 
@@ -2003,7 +2091,7 @@ papplSystemSetMaxImageSize(
       max_size = 16 * 1024 * 1024;
 
 #else
-    struct rlimit	limit;		// Memmory limits
+    struct rlimit	limit;		// Memory limits
 
     if (getrlimit(RLIMIT_DATA, &limit))
       max_size = 16 * 1024 * 1024;
@@ -2051,13 +2139,13 @@ papplSystemSetMaxLogSize(
 {
   if (system)
   {
-    _papplRWLockWrite(system);
+    pthread_mutex_lock(&system->log_mutex);
 
-    system->logmaxsize = maxsize;
+    system->log_max_size = maxsize;
 
     _papplSystemConfigChanged(system);
 
-    _papplRWUnlock(system);
+    pthread_mutex_unlock(&system->log_mutex);
   }
 }
 
@@ -2501,9 +2589,9 @@ add_listeners(
   if ((addrlist = httpAddrGetList(name, family, service)) == NULL)
   {
     if (name && *name == '/')
-      papplLog(system, PAPPL_LOGLEVEL_INFO, "Unable to lookup address(es) for '%s': %s", name, cupsLastErrorString());
+      papplLog(system, PAPPL_LOGLEVEL_INFO, "Unable to lookup address(es) for '%s': %s", name, cupsGetErrorString());
     else
-      papplLog(system, PAPPL_LOGLEVEL_INFO, "Unable to lookup address(es) for '%s:%d': %s", name ? name : "*", port, cupsLastErrorString());
+      papplLog(system, PAPPL_LOGLEVEL_INFO, "Unable to lookup address(es) for '%s:%d': %s", name ? name : "*", port, cupsGetErrorString());
   }
   else
   {
@@ -2516,9 +2604,9 @@ add_listeners(
 	if (system->port)
 	{
 	  if (name && *name == '/')
-	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s': %s", name, cupsLastErrorString());
+	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s': %s", name, cupsGetErrorString());
 	  else
-	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s:%d': %s", httpAddrGetString(&addr->addr, temp, (cups_len_t)sizeof(temp)), system->port, cupsLastErrorString());
+	    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create listener socket for '%s:%d': %s", httpAddrGetString(&addr->addr, temp, (cups_len_t)sizeof(temp)), system->port, cupsGetErrorString());
 	}
       }
       else
