@@ -1,16 +1,12 @@
 //
 // System load/save functions for the Printer Application Framework
 //
-// Copyright © 2020-2022 by Michael R Sweet.
+// Copyright © 2020-2024 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
 //
 
-//
-// Include necessary headers...
-//
-
 #include "pappl-private.h"
 
 
@@ -72,7 +68,7 @@ papplSystemLoadState(
   if ((fp = cupsFileOpen(filename, "r")) == NULL)
   {
     if (errno != ENOENT)
-      papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to open system state file '%s': %s", filename, cupsLastErrorString());
+      papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to open system state file '%s': %s", filename, cupsGetErrorString());
 
     return (false);
   }
@@ -134,13 +130,14 @@ papplSystemLoadState(
       cups_option_t	*options = NULL;// Options
       const char	*printer_id,	// Printer ID
 			*printer_name,	// Printer name
+			*printer_state,	// Printer state
 			*device_id,	// Device ID
 			*device_uri,	// Device URI
 			*driver_name;	// Driver name
       pappl_printer_t	*printer;	// Current printer
 
 
-      if ((num_options = cupsParseOptions(value, 0, &options)) != 5 || (printer_id = cupsGetOption("id", num_options, options)) == NULL || strtol(printer_id, NULL, 10) <= 0 || (printer_name = cupsGetOption("name", num_options, options)) == NULL || (device_id = cupsGetOption("did", num_options, options)) == NULL || (device_uri = cupsGetOption("uri", num_options, options)) == NULL || (driver_name = cupsGetOption("driver", num_options, options)) == NULL)
+      if ((num_options = cupsParseOptions(value, /*end*/NULL, 0, &options)) == 0 || (printer_id = cupsGetOption("id", num_options, options)) == NULL || strtol(printer_id, NULL, 10) <= 0 || (printer_name = cupsGetOption("name", num_options, options)) == NULL || (device_id = cupsGetOption("did", num_options, options)) == NULL || (device_uri = cupsGetOption("uri", num_options, options)) == NULL || (driver_name = cupsGetOption("driver", num_options, options)) == NULL)
       {
         papplLog(system, PAPPL_LOGLEVEL_ERROR, "Bad printer definition on line %d of '%s'.", linenum, filename);
         break;
@@ -156,6 +153,9 @@ papplSystemLoadState(
 	  papplLog(system, PAPPL_LOGLEVEL_ERROR, "Dropping printer '%s' and its job history because an error occurred: %s", printer_name, strerror(errno));
       }
 
+      if ((system->options & PAPPL_SOPTIONS_MULTI_QUEUE) && (printer_state = cupsGetOption("state", num_options, options)) != NULL && (ipp_pstate_t)atoi(printer_state) == IPP_PSTATE_STOPPED)
+        papplPrinterPause(printer);
+
       while (read_line(fp, line, sizeof(line), &value, &linenum))
       {
         if (!strcasecmp(line, "</Printer>"))
@@ -288,7 +288,7 @@ papplSystemLoadState(
 			*job_format,	// Job format
 			*job_value;	// Job option value
 
-	  num_options = cupsParseOptions(value, 0, &options);
+	  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
 	  if ((job_id = cupsGetOption("id", num_options, options)) == NULL || strtol(job_id, NULL, 10) <= 0 || (job_name = cupsGetOption("name", num_options, options)) == NULL || (job_username = cupsGetOption("username", num_options, options)) == NULL || (job_format = cupsGetOption("format", num_options, options)) == NULL)
 	  {
@@ -409,7 +409,7 @@ papplSystemSaveState(
 
   if ((fp = cupsFileOpen(filename, "w")) == NULL)
   {
-    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create system state file '%s': %s", filename, cupsLastErrorString());
+    papplLog(system, PAPPL_LOGLEVEL_ERROR, "Unable to create system state file '%s': %s", filename, cupsGetErrorString());
     return (false);
   }
 
@@ -451,17 +451,23 @@ papplSystemSaveState(
 
     printer = (pappl_printer_t *)cupsArrayGetElement(system->printers, i);
 
-    if (printer->is_deleted)
-      continue;
-
     _papplRWLockRead(printer);
 
+    if (printer->is_deleted)
+    {
+      _papplRWUnlock(printer);
+      continue;
+    }
+
     num_options = cupsAddIntegerOption("id", printer->printer_id, num_options, &options);
     num_options = cupsAddOption("name", printer->name, num_options, &options);
     num_options = cupsAddOption("did", printer->device_id ? printer->device_id : "", num_options, &options);
     num_options = cupsAddOption("uri", printer->device_uri, num_options, &options);
     num_options = cupsAddOption("driver", printer->driver_name, num_options, &options);
 
+    if (system->options & PAPPL_SOPTIONS_MULTI_QUEUE)
+      num_options = cupsAddIntegerOption("state", (int)printer->state, num_options, &options);
+
     write_options(fp, "<Printer", num_options, options);
     cupsFreeOptions(num_options, options);
 
@@ -494,7 +500,6 @@ papplSystemSaveState(
       cupsFilePrintf(fp, "label-tear-offset-configured %d\n", printer->driver_data.tear_offset_configured);
 
     write_media_col(fp, "media-col-default", &printer->driver_data.media_default);
-
     for (j = 0; j < (cups_len_t)printer->driver_data.num_source; j ++)
     {
       if (printer->driver_data.media_ready[j].size_name[0])
@@ -505,10 +510,12 @@ papplSystemSaveState(
         write_media_col(fp, name, printer->driver_data.media_ready + j);
       }
     }
+
     if (printer->driver_data.orient_default)
       cupsFilePutConf(fp, "orientation-requested-default", ippEnumString("orientation-requested", (int)printer->driver_data.orient_default));
     if (printer->driver_data.bin_default && printer->driver_data.num_bin > 0)
       cupsFilePutConf(fp, "output-bin-default", printer->driver_data.bin[printer->driver_data.bin_default]);
+
     if (printer->driver_data.color_default)
       cupsFilePutConf(fp, "print-color-mode-default", _papplColorModeString(printer->driver_data.color_default));
     if (printer->driver_data.content_default)
@@ -519,12 +526,16 @@ papplSystemSaveState(
       cupsFilePutConf(fp, "print-quality-default", ippEnumString("print-quality", (int)printer->driver_data.quality_default));
     if (printer->driver_data.scaling_default)
       cupsFilePutConf(fp, "print-scaling-default", _papplScalingString(printer->driver_data.scaling_default));
-    if (printer->driver_data.darkness_configured)
+    if (printer->driver_data.speed_supported[1] > 0)
+      cupsFilePrintf(fp, "print-speed-default %d\n", printer->driver_data.speed_default);
+    if (printer->driver_data.darkness_supported > 0)
       cupsFilePrintf(fp, "printer-darkness-configured %d\n", printer->driver_data.darkness_configured);
-    if (printer->driver_data.sides_default)
-      cupsFilePutConf(fp, "sides-default", _papplSidesString(printer->driver_data.sides_default));
     if (printer->driver_data.x_default)
       cupsFilePrintf(fp, "printer-resolution-default %dx%ddpi\n", printer->driver_data.x_default, printer->driver_data.y_default);
+
+    if (printer->driver_data.sides_default)
+      cupsFilePutConf(fp, "sides-default", _papplSidesString(printer->driver_data.sides_default));
+
     for (j = 0; j < (cups_len_t)printer->driver_data.num_vendor; j ++)
     {
       char	defname[128],		// xxx-default name
@@ -633,7 +644,7 @@ add_time(const char    *name,		// I  - Name
   // Format the number as a long integer...
   snprintf(buffer, sizeof(buffer), "%ld", (long)value);
 
-  // Add the option wih the string...
+  // Add the option with the string...
   return (cupsAddOption(name, buffer, num_options, options));
 }
 
@@ -653,7 +664,7 @@ parse_contact(char            *value,	// I - Value
 
 
   memset(contact, 0, sizeof(pappl_contact_t));
-  num_options = cupsParseOptions(value, 0, &options);
+  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
   for (i = num_options, option = options; i > 0; i --, option ++)
   {
@@ -685,7 +696,7 @@ parse_media_col(
 
 
   memset(media, 0, sizeof(pappl_media_col_t));
-  num_options = cupsParseOptions(value, 0, &options);
+  num_options = cupsParseOptions(value, /*end*/NULL, 0, &options);
 
   for (i = num_options, option = options; i > 0; i --, option ++)
   {
