@@ -1,8 +1,7 @@
 //
 // Private system header file for the Printer Application Framework
 //
-// Copyright © 2019-2022 by Michael R Sweet.
-// Copyright © 2010-2019 by Apple Inc.
+// Copyright © 2019-2025 by Michael R Sweet.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
 // information.
@@ -69,16 +68,19 @@ struct _pappl_system_s			// System data
   pappl_contact_t	contact;		// "system-contact-col" value
   char			*hostname;		// Published hostname
   int			port;			// Port number, if any
+  bool			is_listenhost;		// Hostname is listen host
   char			*domain_path;		// Domain socket path, if any
   cups_len_t		num_versions;		// Number of "xxx-firmware-yyy" values
   pappl_version_t	versions[10];		// "xxx-firmware-yyy" values
   char			*footer_html;		// Footer HTML for web interface
   char			*server_header;		// Server: header value
   char			*directory;		// Spool directory
-  char			*logfile;		// Log filename, if any
-  int			logfd;			// Log file descriptor, if any
-  pappl_loglevel_t	loglevel;		// Log level
-  size_t		logmaxsize;		// Maximum log file size or `0` for none
+  pthread_mutex_t	log_mutex;		// Log mutex
+  char			*log_file;		// Log filename, if any
+  int			log_fd;			// Log file descriptor, if any
+  pappl_loglevel_t	log_level;		// Log level
+  size_t		log_max_size;		// Maximum log file size or `0` for none
+  bool			log_is_syslog;		// Log to system log?
   char			*subtypes;		// DNS-SD sub-types, if any
   bool			tls_only;		// Only support TLS?
   char			*auth_service;		// PAM authorization service, if any
@@ -86,7 +88,7 @@ struct _pappl_system_s			// System data
   gid_t			admin_gid;		// PAM administrative group ID
   char			*default_print_group;	// Default PAM printing group, if any
   char			session_key[65];	// Session key
-  pthread_rwlock_t	session_rwlock;		// Reader/writer lock for the session key
+  pthread_mutex_t	session_mutex;		// Mutex for the session key
   time_t		session_time;		// Session key time
   cups_len_t		num_listeners;		// Number of listener sockets
   struct pollfd		listeners[_PAPPL_MAX_LISTENERS];
@@ -172,18 +174,29 @@ extern void		_papplSystemAddLoc(pappl_system_t *system, pappl_loc_t *loc) _PAPPL
 extern void		_papplSystemAddPrinter(pappl_system_t *system, pappl_printer_t *printer, int printer_id) _PAPPL_PRIVATE;
 extern void		_papplSystemAddPrinterIcons(pappl_system_t *system, pappl_printer_t *printer) _PAPPL_PRIVATE;
 extern bool		_papplSystemAddSubscription(pappl_system_t *system, pappl_subscription_t *sub, int sub_id) _PAPPL_PRIVATE;
+
 extern void		_papplSystemCleanSubscriptions(pappl_system_t *system, bool clean_all) _PAPPL_PRIVATE;
 extern void		_papplSystemConfigChanged(pappl_system_t *system) _PAPPL_PRIVATE;
+
 extern void		_papplSystemExportVersions(pappl_system_t *system, ipp_t *ipp, ipp_tag_t group_tag, cups_array_t *ra);
+
 extern _pappl_mime_filter_t *_papplSystemFindMIMEFilter(pappl_system_t *system, const char *srctype, const char *dsttype) _PAPPL_PRIVATE;
 extern _pappl_resource_t *_papplSystemFindResourceForLanguage(pappl_system_t *system, const char *language) _PAPPL_PRIVATE;
 extern _pappl_resource_t *_papplSystemFindResourceForPath(pappl_system_t *system, const char *path) _PAPPL_PRIVATE;
+
+extern bool		_papplSystemIsShutdownNoLock(pappl_system_t *system) _PAPPL_PRIVATE;
+
 extern char		*_papplSystemMakeUUID(pappl_system_t *system, const char *printer_name, int job_id, char *buffer, size_t bufsize) _PAPPL_PRIVATE;
+
 extern void		_papplSystemNeedClean(pappl_system_t *system) _PAPPL_PRIVATE;
+
 extern void		_papplSystemProcessIPP(pappl_client_t *client) _PAPPL_PRIVATE;
+
 extern bool		_papplSystemRegisterDNSSDNoLock(pappl_system_t *system) _PAPPL_PRIVATE;
+
 extern void		_papplSystemSetHostNameNoLock(pappl_system_t *system, const char *value) _PAPPL_PRIVATE;
 extern void		_papplSystemStatusUI(pappl_system_t *system) _PAPPL_PRIVATE;
+
 extern void		_papplSystemUnregisterDNSSDNoLock(pappl_system_t *system) _PAPPL_PRIVATE;
 
 extern void		_papplSystemWebAddPrinter(pappl_client_t *client, pappl_system_t *system) _PAPPL_PRIVATE;
